----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    09:35:55 06/04/2007 
-- Design Name: 
-- Module Name:    dvi_out - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;

library work;
USE work.RLEPatternPkg.ALL;

entity tpg_core is
generic 
(
   C_FAMILY          :  string   := "virtex5";
   C_Chroma_Format   :  integer  := 0 -- 0 = RGB444; 1 = YCbCr422
);
port 
(
   clk                  :  in    std_logic;
   rst                  :  in    std_logic;
   vsync_in             :  in    std_logic;
   hsync_in             :  in    std_logic;
   vblank_in            :  in    std_logic;
   hblank_in            :  in    std_logic;
   CbCrPolarity         :  in    std_logic;
   VSyncPolarity        :  in    std_logic;
   HSyncPolarity        :  in    std_logic;
   VBlankPolarity       :  in    std_logic;
   HBlankPolarity       :  in    std_logic;
   de_in                :  in    std_logic;
   red_in               :  in    std_logic_vector(7 downto 0);
   green_in             :  in    std_logic_vector(7 downto 0);
   blue_in              :  in    std_logic_vector(7 downto 0);
   Motion               :  in    std_logic;
   Motion_speed         :  in    std_logic_vector(7 downto 0);
   PatternSel           :  in    std_logic_vector(3 downto 0);
   EnableXHairs         :  in    std_logic;
   EnableStuckPixel     :  in    std_logic;
   EnableNoise          :  in    std_logic;
   NoiseGain            :  in    std_logic_vector(7 downto 0);
   EnableBox            :  in    std_logic;
   StuckPixelThreshold  :  in    std_logic_vector(15 downto 0);
   ComponentMask        :  in    std_logic_vector(2 downto 0);
   XHairsV              :  in    std_logic_vector(11 downto 0);
   XHairsH              :  in    std_logic_vector(11 downto 0);
   BoxSize              :  in    std_logic_vector(11 downto 0);
   BoxColour            :  in    std_logic_vector(23 downto 0);
   active_line_length   :  in    std_logic_vector(11 downto 0);
   active_frame_height  :  in    std_logic_vector(11 downto 0);
   ZPlateHDeltaStart    :  in    std_logic_vector(15 downto 0);
   ZPlateHDelta2        :  in    std_logic_vector(15 downto 0);
   ZPlateVDeltaStart    :  in    std_logic_vector(15 downto 0);
   ZPlateVDelta2        :  in    std_logic_vector(15 downto 0);
   vsync_out            :  out   std_logic;
   hsync_out            :  out   std_logic;
   vblank_out           :  out   std_logic;
   hblank_out           :  out   std_logic;
   de_out               :  out   std_logic;
   red_out              :  out   std_logic_vector(7 downto 0);
   green_out            :  out   std_logic_vector(7 downto 0);
   blue_out             :  out   std_logic_vector(7 downto 0);
   ZP_debug             :  out   std_logic_vector(57 downto 0);
   TPG_debug            :  out   std_logic_vector(38 downto 0);
   RLHeight             :  out   std_logic_vector(11 downto 0);
   version              :  out   std_logic_vector(31 downto 0)

);
end tpg_core;

architecture Behavioral of tpg_core is
   
component version_reg
  
  generic (
    VERSION_MAJOR     : std_logic_vector (3 downto 0) := X"1";   -- Major Version number
    VERSION_MINOR     : std_logic_vector (7 downto 0) := X"00";  -- Minor Version Number
    VERSION_REVISION  : std_logic_vector (3 downto 0) := X"a";   -- Version Revision character (EDK)
    REVISION_NUMBER   : string := "Build Number: 0000"           -- Internal build number
    );
  port (
    version : out std_logic_vector (31 downto 0)  -- Version number of IP output
    );
end component;


signal   d1                :  STD_LOGIC_VECTOR (11 downto 0);
signal   d2                :  STD_LOGIC_VECTOR (11 downto 0);
signal   hdata             :  STD_LOGIC_VECTOR (7 downto 0);
signal   vdata             :  STD_LOGIC_VECTOR (7 downto 0);
signal   HCount            :  STD_LOGIC_VECTOR (11 downto 0);
signal   d_HCount          :  STD_LOGIC_VECTOR (11 downto 0);
signal   d2_HCount         :  STD_LOGIC_VECTOR (11 downto 0);
signal   d3_HCount         :  STD_LOGIC_VECTOR (11 downto 0);
signal   d4_HCount         :  STD_LOGIC_VECTOR (11 downto 0);
signal   d5_HCount         :  STD_LOGIC_VECTOR (11 downto 0);
signal   VCount            :  STD_LOGIC_VECTOR (11 downto 0);
signal   d_vsync_in        :  std_logic;
signal   d_vblank_in       :  std_logic;
signal   d2_vsync_in       :  std_logic;
signal   re_vsync_in       :  std_logic;
signal   d_re_vsync_in       :  std_logic;
signal   d2_re_vsync_in       :  std_logic;
signal   d_hsync_in        :  std_logic;
signal   d_hblank_in       :  std_logic;
signal   d2_hsync_in       :  std_logic;
signal   d2_de_in          :  std_logic;
signal   d3_de_in          :  std_logic;
signal   fe_de_in          :  std_logic;
signal   d_fe_de_in        :  std_logic;
signal   d2_fe_de_in       :  std_logic;
signal   re_de_in          :  std_logic;
signal   d_de_in           :  std_logic;
signal   re_hsync_in       :  std_logic;
signal   fe_hsync_in       :  std_logic;
signal   RampStart         :  STD_LOGIC_VECTOR (7 downto 0);


signal   red               :  STD_LOGIC_VECTOR ( 7 downto 0);
signal   green             :  STD_LOGIC_VECTOR ( 7 downto 0);
signal   blue              :  STD_LOGIC_VECTOR ( 7 downto 0);
signal   Y                 :  STD_LOGIC_VECTOR ( 7 downto 0);
signal   Cb                :  STD_LOGIC_VECTOR ( 7 downto 0);
signal   Cr                :  STD_LOGIC_VECTOR ( 7 downto 0);
signal   bar_red           :  STD_LOGIC_VECTOR ( 7 downto 0);
signal   bar_green         :  STD_LOGIC_VECTOR ( 7 downto 0);
signal   bar_blue          :  STD_LOGIC_VECTOR ( 7 downto 0);
signal   bar_Y             :  STD_LOGIC_VECTOR ( 7 downto 0);
signal   bar_Cb            :  STD_LOGIC_VECTOR ( 7 downto 0);
signal   bar_Cr            :  STD_LOGIC_VECTOR ( 7 downto 0);
signal   BarWidth          :  STD_LOGIC_VECTOR ( 8 downto 0);
signal   TartanBarHeight   :  STD_LOGIC_VECTOR ( 8 downto 0);
signal   BarHCount         :  STD_LOGIC_VECTOR ( 8 downto 0);
signal   HBarSel           :  STD_LOGIC_VECTOR ( 2 downto 0);
signal   BarSel            :  STD_LOGIC_VECTOR ( 2 downto 0);
signal   XHairValue        :  STD_LOGIC_VECTOR ( 7 downto 0);
signal   CSel              :  std_logic;
signal   ZPlateDOut        :  STD_LOGIC_VECTOR ( 7 downto 0);
signal   ZPStart           :  STD_LOGIC_VECTOR ( 15 downto 0);
signal   d_red_in          :  STD_LOGIC_VECTOR ( 7 downto 0);
signal   d_green_in        :  STD_LOGIC_VECTOR ( 7 downto 0);
signal   d_blue_in         :  STD_LOGIC_VECTOR ( 7 downto 0);
signal   BoxHCoord         :  STD_LOGIC_VECTOR (11 downto 0);
signal   BoxVCoord         :  STD_LOGIC_VECTOR (11 downto 0);
signal   HMin              :  STD_LOGIC_VECTOR (11 downto 0);
signal   HMax              :  STD_LOGIC_VECTOR (11 downto 0);
signal   VMin              :  STD_LOGIC_VECTOR (11 downto 0);
signal   VMax              :  STD_LOGIC_VECTOR (11 downto 0);
signal   BoxTop            :  STD_LOGIC_VECTOR (11 downto 0);
signal   BoxBottom         :  STD_LOGIC_VECTOR (11 downto 0);
signal   BoxLeft           :  STD_LOGIC_VECTOR (11 downto 0);
signal   BoxRight          :  STD_LOGIC_VECTOR (11 downto 0);
signal   BoxEn             :  std_logic;
signal   HBoxEn            :  std_logic;
signal   VBoxEn            :  std_logic;
signal   HDir              :  std_logic;
signal   VDir              :  std_logic;
signal   t_red_out         :  std_logic_vector(7 downto 0);
signal   t_blue_out        :  std_logic_vector(7 downto 0);
signal   t_green_out       :  std_logic_vector(7 downto 0);
signal   t_de_out          :  std_logic;
signal   d_de_out          :  std_logic;
signal   zp_vsync_in       :  std_logic;
signal   zp_hsync_in       :  std_logic;
signal   EnableTartan      :  std_logic;
signal   TartanBarVIndexOffset   :  integer := 0;
signal   TartanBarIndex          :  integer := 0;
signal   TartanBarSel      		:  std_logic_vector(2 downto 0);
signal   TartanBarVCount         :  std_logic_vector(8 downto 0);
signal   TartanBarVSel           :  std_logic_vector(2 downto 0);
signal   XHatchV                 :  std_logic;
signal   XHatchH                 :  std_logic;
signal   LastLine         	      :  std_logic_vector(11 downto 0);
signal   LastPixel       	      :  std_logic_vector(11 downto 0);
signal   RL_Y                    :  std_logic_vector(7 downto 0);
signal   RL_Cr                   :  std_logic_vector(7 downto 0);
signal   RL_Cb                   :  std_logic_vector(7 downto 0);
signal   UseStuckPixel           :  std_logic;
signal   StuckPixelR             :  std_logic_vector(5 downto 0);
signal   StuckPixelG             :  std_logic_vector(5 downto 0);
signal   StuckPixelB             :  std_logic_vector(5 downto 0);
signal   PixRepeatCount   	      :  std_logic_vector(11 downto 0);
signal   LineRepeatCount  	      :  std_logic_vector(11 downto 0);
signal   RLInstrAddr      	      :  std_logic_vector(4 downto 0);
signal   RLDataAddr              :  std_logic_vector(4 downto 0);
signal   e_RegisterRLWidth       :  std_logic;
signal   RegisterRLWidth         :  std_logic;
signal   RegisterRLHeight        :  std_logic;
signal   d_RegisterRLHeight      :  std_logic;
signal   LineRepeatCountEn       :  std_logic;
   
--signal   RLHeight                :  std_logic_vector(11 downto 0);
signal   RLWidth                 :  std_logic_vector(11 downto 0);
signal   RLWidthMinus1           :  std_logic_vector(11 downto 0);
signal   RLInstrValue            :  std_logic_vector(15 downto 0);
signal   RLDataValue             :  std_logic_vector(15 downto 0);
signal   RLInstrAddrPreset       :  std_logic_vector(4 downto 0);
signal   RLDataAddrPreset        :  std_logic_vector(4 downto 0);
signal   PRNG1Out                :  std_logic_vector(33 downto 0);
signal   PRNG2Out                :  std_logic_vector(33 downto 0);

signal   ExtendedNoiseGain       :  std_logic_vector(9 downto 0);
signal   ExtendedRedNoise        :  std_logic_vector(9 downto 0);
signal   ExtendedGreenNoise      :  std_logic_vector(9 downto 0);
signal   ExtendedBlueNoise       :  std_logic_vector(9 downto 0);
signal   RedNoise                :  std_logic_vector(19 downto 0);
signal   GreenNoise              :  std_logic_vector(19 downto 0);
signal   BlueNoise               :  std_logic_vector(19 downto 0);
signal   NoisyRedOut             :  std_logic_vector(9 downto 0);
signal   NoisyGreenOut           :  std_logic_vector(9 downto 0);
signal   NoisyBlueOut            :  std_logic_vector(9 downto 0);
signal	PRNG2PrstEn					:  std_logic;

type     integer_array IS ARRAY ( NATURAL RANGE <>) OF integer;
constant Tartan_bars_array : integer_array := (4,2,6,0,7,1,5,3,2,7,3,1,6,4,0,5,6,3,0,2,5,7,4,1,0,1,2,3,4,5,6,7,7,6,5,4,3,2,1,0,1,4,7,5,2,0,3,6,5,0,4,6,1,3,7,2,3,5,1,7,0,6,2,4);
constant	PRNG1Width			:	integer := 34;
constant	PRNG1Seed			:	std_logic_vector((PRNG1Width-1) downto 0) := "0110100110101110001011111010010101";
constant	PRNG2Width			:	integer := 34;
constant	PRNG2Seed			:	std_logic_vector((PRNG1Width-1) downto 0) := "0110100110101110001011111010010101";



begin
TPGCount : process (clk)   
begin
if (clk'event) and (clk = '1') then
   
   if (VSyncPolarity = '0') then
      d_vsync_in     <= vsync_in;
   else
      d_vsync_in     <= not(vsync_in);
   end if;
   if (HSyncPolarity = '0') then
      d_hsync_in     <= hsync_in;
   else
      d_hsync_in     <= not(hsync_in);
   end if;
   if (VBlankPolarity = '0') then
      d_vblank_in     <= vblank_in;
   else
      d_vblank_in     <= not(vblank_in);
   end if;
   if (HBlankPolarity = '0') then
      d_hblank_in     <= hblank_in;
   else
      d_hblank_in     <= not(hblank_in);
   end if;
      
   d2_hsync_in    <= d_hsync_in;
   d2_vsync_in    <= d_vsync_in;
   re_vsync_in    <= d_vsync_in and not(d2_vsync_in);
   d_re_vsync_in  <= re_vsync_in;
   d2_re_vsync_in <= d_re_vsync_in;
   re_hsync_in    <= d_hsync_in and not(d2_hsync_in);
   fe_hsync_in    <= d2_hsync_in and not(d_hsync_in);
   d_fe_de_in     <= fe_de_in;
   d2_fe_de_in    <= d_fe_de_in;
   
   if (rst = '1') then
      RampStart   <= (others => '0');
      vdata       <= (others => '0');
      hdata       <= (others => '0');
      VCount      <= (others => '0');
      HCount      <= (others => '0');
   else
      if (re_vsync_in = '1') then
         if (Motion = '1') then
            RampStart   <= RampStart + Motion_speed;
         else
            RampStart   <= "00000000";
         end if;
         vdata <= RampStart;
         hdata <= (others => '0');
      else
         if (d2_fe_de_in = '1') then
            hdata    <= RampStart;
            vdata    <= vdata + 1;
         elsif (d_de_in = '1') then
            hdata    <= hdata + 1;
         end if;
      end if;
   
      if (re_vsync_in = '1') then
         VCount      <= (others => '0');
         HCount      <= (others => '0');
      else
         if (d2_fe_de_in = '1') then
            VCount   <= VCount + 1;
            HCount   <= (others => '0');
         elsif (d_de_in = '1') then
            HCount   <= HCount + 1;
         end if;
      end if;
      d_HCount    <= HCount;
      d2_HCount   <= d_HCount;
      d3_HCount   <= d2_HCount;
      d4_HCount   <= d3_HCount;
      d5_HCount   <= d4_HCount;
   end if;
end if;
end process;   

ZPStart  <= RampStart & "00000000";


ZPSyncSel:process(hsync_in, vsync_in, VSyncPolarity, HSyncPolarity)
begin
   if (VSyncPolarity = '0') then
      zp_vsync_in <= vsync_in;
   else
      zp_vsync_in <= not(vsync_in);
   end if;
   if (HSyncPolarity = '0') then
      zp_hsync_in <= hsync_in;
   else
      zp_hsync_in <= not(hsync_in);
   end if;
end process;

-- PRNG1 resets every frame, for stuck-pixel reasons.
PRNG1: entity work.prng
generic map
(
   width          => PRNG1Width
)
port  map
(
   clk         => clk,
   seed        => PRNG1Seed,
   random_num  => PRNG1Out,
   prst        => re_vsync_in
);

-- PRNG2 is free-running.
PRNG2: entity work.prng
generic map
(
   width          => PRNG2Width
)
port  map
(
   clk         => clk,
   seed        => PRNG2Seed,
   random_num  => PRNG2Out,
   prst        => PRNG2PrstEn
);


StuckPixelGen : process (clk)   
begin
if (clk'event) and (clk = '1') then
	PRNG2PrstEn	<= not(EnableNoise);
   StuckPixelR <= PRNG1Out(21 downto 16);
   StuckPixelG <= PRNG1Out(27 downto 22);
   StuckPixelB <= PRNG1Out(33 downto 28);
   if (PRNG1Out(15 downto 0) < StuckPixelThreshold) then
      UseStuckPixel <= '1';
   else
      UseStuckPixel <= '0';
   end if;
end if;
end process;   

ZPlate1: entity work.zplate
generic map
(
   OutWidth       => 8,
   DeltaWidth     => 16
   
)
port  map
(
   clk   	      => clk,
   HDeltaStart    => ZPlateHDeltaStart,
   HDelta2        => ZPlateHDelta2,
   VDeltaStart    => ZPlateVDeltaStart,
   VDelta2        => ZPlateVDelta2,
   ZPStart        => ZPStart,
   de_in          => de_in,
   HSync          => zp_hsync_in,
   VSync          => zp_vsync_in,
   Dout           => ZPlateDOut,
   ZP_debug       => ZP_debug
);


TartanBarVIndexOffset   <= conv_integer('0' &(TartanBarVSel & "000"));
TartanBarIndex          <= TartanBarVIndexOffset + conv_integer('0' & HBarSel);
TartanBarSel      		<= conv_std_logic_vector(Tartan_bars_array(TartanBarIndex), 3);


BarsGen : process (clk)   
begin
if (clk'event) and (clk = '1') then
   BarWidth          <= (active_line_length(11 downto 3) - 1); 
   TartanBarHeight   <= (active_frame_height(11 downto 3) - 1); 
   LastLine          <= active_frame_height - 1;
   LastPixel         <= active_line_length - 1;
   if (rst = '1') then
      TartanBarVSel     <= (others => '0');
      TartanBarVCount   <= (others => '0');
   else
      if (re_vsync_in = '1') then
         TartanBarVCount   <= (others => '0');
         TartanBarVSel     <= (others => '0');
      elsif (d2_fe_de_in = '1') then
         if (TartanBarVCount = TartanBarHeight) then
            TartanBarVSel     <= TartanBarVSel + '1';
            TartanBarVCount   <= (others => '0');
         else
            TartanBarVCount   <= TartanBarVCount + '1';
         end if;
      end if;
   end if;


   if (rst = '1') then
      HBarSel     <= (others => '0');
      BarHCount   <= "000000001"; -- Cheat
   else
      if (d2_fe_de_in = '1') then
         BarHCount         <= "000000001"; -- Cheat
         HBarSel           <= (others => '0');
      elsif (BarHCount = BarWidth) then
         HBarSel           <= HBarSel + '1';
         BarHCount         <= (others => '0');
      elsif (d_de_in = '1') then
         BarHCount         <= BarHCount + '1';
      end if;
   end if;
   
   if (re_vsync_in = '1') then
      XHatchV           <= '0';
   elsif ((TartanBarVCount = "000000000") or (VCount = LastLine)) then
      XHatchV           <= '1';
   else
      XHatchV           <= '0';
   end if;

   if ((HCount = "000000000000") or (HCount = LastPixel)) then
      XHatchH           <= '1';
   elsif (d_de_in = '1') then
      if (BarHCount = BarWidth) then
         XHatchH        <= '1';
      else
         XHatchH        <= '0';
      end if;
   else
      XHatchH           <= '0';
   end if;

   if (EnableTartan = '0') then
      BarSel   <= HBarSel;
   else
      BarSel   <= TartanBarSel;
   end if;
   
   case BarSel is
      when "000" => --  white
         bar_red      <= "11111111";
         bar_green    <= "11111111";
         bar_blue     <= "11111111";
         bar_Y        <= "11101011"; 
         bar_Cb       <= "10000000"; 
         bar_Cr       <= "10000000";
      when "001" => --  yellow
         bar_red      <= "11111111";
         bar_green    <= "11111111";
         bar_blue     <= "00000000";
         bar_Y        <= "11010010"; 
         bar_Cb       <= "00001000"; 
         bar_Cr       <= "10010010";
      when "010" => --  cyan
         bar_red      <= "00000000";
         bar_green    <= "11111111";
         bar_blue     <= "11111111";
         bar_Y        <= "10101010"; 
         bar_Cb       <= "10100110"; 
         bar_Cr       <= "00001000";
      when "011" => --  green
         bar_red      <= "00000000";
         bar_green    <= "11111111";
         bar_blue     <= "00000000";
         bar_Y        <= "10010001"; 
         bar_Cb       <= "00110110"; 
         bar_Cr       <= "00100010";
      when "100" => --  magenta
         bar_red      <= "11111111";
         bar_green    <= "00000000";
         bar_blue     <= "11111111";
         bar_Y        <= "01101010"; 
         bar_Cb       <= "11001010"; 
         bar_Cr       <= "11011110";
      when "101" => --  red
         bar_red      <= "11111111";
         bar_green    <= "00000000";
         bar_blue     <= "00000000";
         bar_Y        <= "01010001"; 
         bar_Cb       <= "01011010"; 
         bar_Cr       <= "11110000";
      when "110" => --  blue
         bar_red      <= "00000000";
         bar_green    <= "00000000";
         bar_blue     <= "11111111";
         bar_Y        <= "00101001"; 
         bar_Cb       <= "11110000"; 
         bar_Cr       <= "01101110";
      when "111" => --  black
         bar_red      <= "00000000";
         bar_green    <= "00000000";
         bar_blue     <= "00000000";
         bar_Y        <= "00001000"; 
         bar_Cb       <= "10000000"; 
         bar_Cr       <= "10000000";
      when others => null;
   end case;
end if;
end process;  



TPG_SM : process (clk)   
begin
if (clk'event) and (clk = '1') then
   d_de_in        <= de_in;
   d2_de_in       <= d_de_in;
   d3_de_in       <= d2_de_in;
   d_red_in       <= red_in;
   d_green_in     <= green_in;
   d_blue_in      <= blue_in;
   
   vsync_out      <= d_vsync_in;
   hsync_out      <= d_hsync_in;
   vblank_out     <= d_vblank_in;
   hblank_out     <= d_hblank_in;
   
   t_de_out       <= d3_de_in;
   fe_de_in       <= not(d2_de_in) and d3_de_in;
   EnableTartan   <= '0'; -- Cheat.
   
   case (PatternSel) is
      when "0000" => -- Input passthrough
         XHairValue <= "11110000"; -- max
         red   <= red_in;
         green <= green_in;
         blue  <= blue_in;
         Y     <= red_in;
         if (CbCrPolarity = '0') then
            Cb    <= green_in;
            Cr    <= green_in;
         else
            Cb    <= d_green_in;
            Cr    <= d_green_in;
         end if;
      when "0001" => -- horizontal ramp
         XHairValue <= "11110000"; -- max
         red   <= hdata;
         green <= hdata;
         blue  <= hdata;
         Y     <= hdata;
         Cb    <= hdata; 
         Cr    <= hdata; 
      when "0010" => -- vertical ramp
         XHairValue <= "11110000"; -- max
         red   <= vdata;
         green <= vdata;
         blue  <= vdata;
         Y     <= vdata;
         Cb    <= vdata;
         Cr    <= vdata;
      when "0011" => -- temporal ramp
         XHairValue <= "11110000"; -- max
         red   <= RampStart;
         green <= RampStart;
         blue  <= RampStart;
         Y     <= RampStart;
         Cb    <= RampStart;
         Cr    <= RampStart;
      when "0100" => -- red
         XHairValue <= "11110000"; -- max
         red   <= "10110100";
         green <= "00010000";
         blue  <= "00010000";
         Y     <= "01010001"; 
         Cb    <= "01011010"; 
         Cr    <= "11110000";
      when "0101" => -- green
         XHairValue <= "11110000"; -- max
         red   <= "00010000";
         green <= "10110100";
         blue  <= "00010000";
         Y     <= "10010001"; 
         Cb    <= "00110110"; 
         Cr    <= "00100010";
      when "0110" => -- blue
         XHairValue <= "11110000"; -- max
         red   <= "00010000";
         green <= "00010000";
         blue  <= "10110100";
         Y     <= "00101001"; 
         Cb    <= "11110000"; 
         Cr    <= "01101110";
      when "0111" => -- black
         XHairValue <= "11110000"; -- max
         red   <= "00010000";
         green <= "00010000";
         blue  <= "00010000";
         Y     <= "00001000"; 
         Cb    <= "10000000"; 
         Cr    <= "10000000";
      when "1000" => -- white
         XHairValue <= "00010000"; -- min
         red   <= "11110000";
         green <= "11110000";
         blue  <= "11110000";
         Y     <= "11101011"; 
         Cb    <= "10000000"; 
         Cr    <= "10000000";
      when "1001" => -- bars
         XHairValue <= "11110000"; -- max
         red   <= bar_red;
         green <= bar_green;
         blue  <= bar_blue;
         Y     <= bar_Y; 
         Cb    <= bar_Cb; 
         Cr    <= bar_Cr;
      when "1010" => -- ZonePlate/Sweep
         XHairValue <= "11110000"; -- max
         red   <= ZPlateDOut;
         green <= ZPlateDOut;
         blue  <= ZPlateDOut;
         Y     <= ZPlateDOut; 
         Cb    <= ZPlateDOut; 
         Cr    <= ZPlateDOut;
      when "1011" => -- tartan bars
         EnableTartan   <= '1';
         XHairValue <= "11110000"; -- max
         red   <= bar_red;
         green <= bar_green;
         blue  <= bar_blue;
         Y     <= bar_Y; 
         Cb    <= bar_Cb; 
         Cr    <= bar_Cr; 
      when "1100" => -- Crosshatch
         XHairValue <= "11110000"; -- max
         if ((XHatchV = '1') or (XHatchH = '1')) then
            red   <= "11110000";
            green <= "11110000";
            blue  <= "11110000";
            Y     <= "11101011"; 
            Cb    <= "10000000"; 
            Cr    <= "10000000"; 
         else
            red   <= "00010000";
            green <= "00010000";
            blue  <= "00010000";
            Y     <= "00001000"; 
            Cb    <= "10000000"; 
            Cr    <= "10000000";
			end if;
      when "1101" => -- User-defined pattern, from RLE decoder
         XHairValue <= "11110000"; -- max
--         red   <= RLE_red;
--         green <= RLE_green;
--         blue  <= RLE_blue;
         Y     <= RL_Y; 
         Cb    <= RL_Cb; 
         Cr    <= RL_Cr; 
      when others => null;
   end case;
end if;
end process;

Select_RLE_Y : if (C_Chroma_Format = 1) generate -- YUV8 case
   -- The TPG subsystem handles the muxing of Cb/Cr
   RL_Y     <= RLDataValue(7 downto 0);
   RL_Cr    <= RLDataValue(15 downto 8);
   RL_Cb    <= RLDataValue(15 downto 8);
end generate;

RLControl: process (clk)             
begin                              
if (clk'event) and (clk = '1') then
   if (rst = '1') then
      RLInstrAddr                <= (others => '0');
      RLDataAddr                 <= (others => '0');
      e_RegisterRLWidth          <= '0';
      RegisterRLHeight           <= '0';
   else
      d_RegisterRLHeight         <= RegisterRLHeight;
      if (re_vsync_in = '1') then
         RLInstrAddr                <= (others => '0'); 
         RLDataAddr                 <= (others => '0');
         RegisterRLHeight           <= '0';     
         e_RegisterRLWidth          <= '0';
      elsif (d_re_vsync_in = '1') then
         RegisterRLHeight           <= '1';     
      else
         if (d2_fe_de_in = '1') then -- Safe to delay this until things calm down after end of line.
            if (LineRepeatCount = "000000000000") then
               -- At the end of a line, and end of VSegment, increment data pointer to the start-address of the NEXT line.
               RLDataAddr           <= RLDataAddr + 1;
            else
               -- At the end of a line, mid-VSegment, reset the data-pointer to the start-address of the same encoded line....
               RLDataAddr           <= RLDataAddrPreset;
            end if;
         elsif (PixRepeatCount = RLWidthMinus1) then
            -- At the end of a HSegment, mid-line, move to next HSeg instruction.
            if (d2_de_in = '1') then
               RLDataAddr              <= RLDataAddr + 1;
            end if;
         elsif (d2_de_in = '1') then
            if (RLDataAddr(0) = '1') then
               RLDataAddr              <= RLDataAddr - 1;
            else
               RLDataAddr              <= RLDataAddr + 1;
            end if;
         end if;

         -- Here's the instruction-memory control block.
         if (RegisterRLHeight = '1') then
            -- At the end of a line, or start of frame, there's always a HSeg instruction after a Line instruction. Register the height and then the width.
            RegisterRLHeight        <= '0';
            e_RegisterRLWidth       <= '1'; 
            RLInstrAddr             <= RLInstrAddr + 1;
            LineRepeatCountEn       <= '0';
         elsif (e_RegisterRLWidth = '1') then
            e_RegisterRLWidth       <= '0'; 
            LineRepeatCountEn       <= '0';
         elsif (d2_fe_de_in = '1') then -- Safe to delay this until things calm down after end of line.
            if (LineRepeatCount = "000000000000") then
               -- At the end of a line, and end of VSegment, increment instruction pointer to the start-address of the NEXT line, and register the height
               RegisterRLHeight     <= '1';
               RLInstrAddr          <= RLInstrAddr + 1;
               LineRepeatCountEn    <= '0';
            else
               -- At the end of a line, mid-VSegment, reset the instruction pointer to the start-address of the same encoded line....
               e_RegisterRLWidth    <= '1';  -- ... and register HSegment width that starts that line
               RLInstrAddr          <= RLInstrAddrPreset;
               LineRepeatCountEn    <= '1';
            end if;
         elsif (PixRepeatCount = RLWidthMinus1) then
            -- At the end of a HSegment, mid-line, move to next HSeg instruction.
            if (d2_de_in = '1') then
               RLInstrAddr             <= RLInstrAddr + '1';
               e_RegisterRLWidth       <= '1'; 
            end if;
            LineRepeatCountEn       <= '0';
         elsif (d2_de_in = '1') then
            e_RegisterRLWidth       <= '0'; 
            LineRepeatCountEn       <= '0';
         else
            e_RegisterRLWidth       <= '0'; 
            LineRepeatCountEn       <= '0';
         end if;
      end if;

      RegisterRLWidth               <= e_RegisterRLWidth;
   end if;

   if (rst = '1') then
      PixRepeatCount             <= (others => '0');
      LineRepeatCount            <= (others => '0');
      RLInstrAddrPreset          <= (others => '0');
      RLDataAddrPreset           <= (others => '0');
   else
      if (RegisterRLHeight = '1') then
         RLHeight             <= RLInstrValue(11 downto 0);
         LineRepeatCount      <= RLInstrValue(11 downto 0) - "000000000001";    -- RLHeight value minus 1
         RLDataAddrPreset     <= RLDataAddr;    
      elsif (LineRepeatCountEn = '1') then
            LineRepeatCount   <= LineRepeatCount - 1;
      end if;

      if (d_RegisterRLHeight = '1') then
         RLInstrAddrPreset    <= RLInstrAddr;   
      end if;
      
      if (RegisterRLWidth = '1') then
         RLWidth              <= RLInstrValue(11 downto 0);
      end if;
      RLWidthMinus1 <= RLWidth - "000000000001";
      
      if (PixRepeatCount = RLWidthMinus1) then
         PixRepeatCount       <= (others => '0'); 
      elsif (d2_de_in = '1') then
         PixRepeatCount    <= PixRepeatCount + 1;
      else
         PixRepeatCount       <= (others => '0');
      end if;
   end if;
end if;
end process;

RLInstrValue   <= Instr_Array(conv_integer('0' & RLInstrAddr));
RLDataValue    <= RLEPattern(conv_integer('0' & RLDataAddr));

BoxGen : process (clk)             
begin                              
if (clk'event) and (clk = '1') then
   if (rst = '1') then
      BoxTop      <= conv_std_logic_vector(50, 12);
      BoxBottom   <= conv_std_logic_vector(81, 12);
      BoxLeft     <= conv_std_logic_vector(50, 12);
      BoxRight    <= conv_std_logic_vector(81, 12);
      HDir        <= '0'; -- Right
      VDir        <= '0'; -- Down
   else
      if (re_vsync_in = '1') then
   
         -- Define extreme locations for top left corner
         HMin  <= conv_std_logic_vector(1, 12);
         VMin  <= conv_std_logic_vector(1, 12);
         HMax  <= active_line_length - BoxSize;
         VMax  <= active_frame_height - BoxSize;
         
         -- Define Box Edge locations
         BoxRight    <= BoxHCoord + BoxSize;
         BoxLeft     <= BoxHCoord;
         BoxBottom   <= BoxVCoord + BoxSize;
         BoxTop      <= BoxVCoord;
         
         -- Change direction at image edges
         if (HDir = '0') then
            if (BoxHCoord > HMax) then
               HDir  <= '1';
            end if;
         else
            if (BoxHCoord < ("000" & Motion_speed & '0')) then
               HDir  <= '0';
            end if;
         end if;
         
         if (VDir = '0') then
            if (BoxVCoord > VMax) then
               VDir  <= '1';
            end if;
         else
            if (BoxVCoord < ("000" & Motion_speed & '0')) then
               VDir  <= '0';
            end if;
         end if;
   
         -- Increment or decrement box coordinates, to enable moving box.
         if (Motion = '1') then
            if (HDir = '0') then
               BoxHCoord   <= BoxHCoord + Motion_speed;
            else
               BoxHCoord   <= BoxHCoord - Motion_speed;
            end if;
            if (VDir = '0') then
               BoxVCoord   <= BoxVCoord + Motion_speed;
            else
               BoxVCoord   <= BoxVCoord - Motion_speed;
            end if;
         end if;
      end if;

      -- Generate Enable signal for switching box on.
      if (re_vsync_in = '1') then
         VBoxEn   <= '0';
      elsif (VCount = BoxTop) then
         VBoxEn   <= '1';
      elsif (VCount = BoxBottom) then
         VBoxEn   <= '0';
      end if;
   
      if (re_hsync_in = '1') then
         HBoxEn   <= '0';
      elsif (HCount = BoxLeft) then
         HBoxEn   <= '1';
      elsif (HCount = BoxRight) then
         HBoxEn   <= '0';
      end if;

      BoxEn <= VBoxEn and HBoxEn and EnableBox;
   
   end if;
end if;
end process;


RGB444OutGen : if (C_Chroma_Format = 0) generate -- RGB case
   OutGen : process (clk)             
   begin                              
   if (clk'event) and (clk = '1') then
      if (BoxEn = '1') then
         t_red_out               <= BoxColour(7 downto 0);
         t_blue_out              <= BoxColour(15 downto 8);
         t_green_out             <= BoxColour(23 downto 16);
      elsif (EnableXHairs = '1') and ((VCount = XHairsV) or (d2_HCount = XHairsH)) then
         t_red_out    <= XHairValue;
         t_green_out  <= XHairValue;
         t_blue_out   <= XHairValue;
      elsif (EnableStuckPixel = '1') and (UseStuckPixel = '1') then
         t_red_out               <= StuckPixelR & "00";
         t_green_out             <= StuckPixelG & "00";
         t_blue_out              <= StuckPixelB & "00";
      else
         if (ComponentMask(0) = '0') then
            t_red_out      <= red;
         else
            t_red_out      <= "00000000";
         end if;

         if (ComponentMask(1) = '0') then
            t_green_out    <= green;
         else
            t_green_out    <= "00000000";
         end if;

         if (ComponentMask(2) = '0') then
            t_blue_out     <= blue;
         else
            t_blue_out     <= "00000000";
         end if;
      end if;
   end if;
   end process;
end generate;

YCbCr422OutGen : if (C_Chroma_Format = 1) generate -- YCbCr case

   OutGen : process (clk)             
   begin                              
   if (clk'event) and (clk = '1') then
      if ((fe_de_in = '1') or (re_vsync_in = '1')) then
         CSel  <= CbCrPolarity;
      elsif (d3_de_in = '1') then
         CSel  <= not(CSel);
      end if;

      if (BoxEn = '1') then
         t_red_out               <= BoxColour(7 downto 0);
         t_blue_out              <= BoxColour(7 downto 0); -- Both R and B become Luma
         if (CSel = '0') then
            t_green_out          <= BoxColour(23 downto 16);
         else
            t_green_out          <= BoxColour(15 downto 8);
         end if;
      elsif (EnableXHairs = '1') and ((VCount = XHairsV) or (d2_HCount = XHairsH)) then
         t_red_out               <= XHairValue;
         t_blue_out              <= XHairValue;
         t_green_out             <= "10000000";
      elsif (EnableStuckPixel = '1') and (UseStuckPixel = '1') then
         t_red_out               <= StuckPixelR & "00";
         t_green_out             <= StuckPixelG & "00";
         t_blue_out              <= StuckPixelB & "00";
      else
      
         if (ComponentMask(0) = '0') then
            t_red_out            <= Y;
            t_blue_out           <= Y;
         else
            t_red_out            <= "00000000";
            t_blue_out           <= "00000000";
         end if;

         if (CSel = '0') then
            if (ComponentMask(1) = '0') then
               t_green_out       <= Cb;
            else
               t_green_out       <= "10000000";
            end if;
         else
            if (ComponentMask(2) = '0') then
               t_green_out       <= Cr;
            else
               t_green_out       <= "10000000";
            end if;
         end if;
      end if;
   end if;
   end process;
end generate;


------------------------------------------------------------------------------
-- Instantiate hardware timeout for evaluation core.
VReg: version_reg 
generic map
(
   VERSION_MAJOR     => X"2",                   -- Major Version number
   VERSION_MINOR     => X"00",                  -- Minor Version Number
   VERSION_REVISION  => X"a",                   -- Version Revision character (EDK)
   REVISION_NUMBER   => "Build Number: 0002"    -- Build number: (only digits are converted to std_logic_vector)
                                                -- Bits 15:12 represent the Coregen patch number  
                                                -- Bits 11:0 represent the internal build number (set at developer's discretion).
)
port map
(
   version => version  -- Version number of IP output
);

-- axi_tpg_v2_00_a:
-- Build 0001: 
--    Added Version Register.
--       This is only for internal usage. There is no register mapping in the pCore wrapper.
-- Build 0002: 
--    Added Stuck-pixel emulator.
--    Added Noise-generator.



NoiseAdder: process (clk)             
begin
   if (clk'event) and (clk = '1') then
   
      d_de_out       <= t_de_out;
      de_out         <= d_de_out;
   
      ExtendedNoiseGain    <= "00" & NoiseGain;
      ExtendedRedNoise     <= PRNG2Out(7) & PRNG2Out(7) & PRNG2Out(7 downto 0);
      ExtendedGreenNoise   <= PRNG2Out(15) & PRNG2Out(15) & PRNG2Out(15 downto 8);
      ExtendedBlueNoise    <= PRNG2Out(23) & PRNG2Out(23) & PRNG2Out(23 downto 16);
      
      if (EnableNoise = '1') then
         RedNoise       <= ExtendedRedNoise * ExtendedNoiseGain;
         GreenNoise     <= ExtendedGreenNoise * ExtendedNoiseGain;
         BlueNoise      <= ExtendedBlueNoise * ExtendedNoiseGain;
      else 
         RedNoise       <= (others => '0');
         GreenNoise     <= (others => '0');
         BlueNoise      <= (others => '0');
      end if;
            
      NoisyRedOut    <= ("00" & t_red_out) + RedNoise(19 downto 10);
      NoisyGreenOut  <= ("00" & t_green_out) + GreenNoise(19 downto 10);
      NoisyBlueOut   <= ("00" & t_blue_out) + BlueNoise(19 downto 10);
      
      if (NoisyRedOut(9) = '1') then -- Clamp
         red_out        <= (others => '0');
      elsif (NoisyRedOut(8) = '1') then -- Clip
         red_out        <= (others => '1');
      else
         red_out        <= NoisyRedOut(7 downto 0);
      end if;

      if (NoisyGreenOut(9) = '1') then -- Clamp
         green_out      <= (others => '0');
      elsif (NoisyGreenOut(8) = '1') then -- Clip
         green_out      <= (others => '1');
      else
         green_out      <= NoisyGreenOut(7 downto 0);
      end if;

      if (NoisyBlueOut(9) = '1') then -- Clamp
         blue_out       <= (others => '0');
      elsif (NoisyBlueOut(8) = '1') then -- Clip
         blue_out       <= (others => '1');
      else
         blue_out       <= NoisyBlueOut(7 downto 0);
      end if;
      
   end if;
end process;

--TPG_debug <=   d_vsync_in & -- 1 bit
--               d_hsync_in & -- 1 bit
--               VCount & -- 12 bits
--               d_red_in & -- 8 bits
--               d_green_in & -- 8 bits
--               d_blue_in & -- 8 bits
--               d_de_in; -- 1 bit
               
TPG_debug <=   d_vsync_in & -- 1 bit
               d_hsync_in & -- 1 bit
               VCount & -- 12 bits
               t_red_out & -- 8 bits
               t_green_out & -- 8 bits
               t_blue_out & -- 8 bits
               t_de_out; -- 1 bit
               

-- 39 bits

end Behavioral;
