-- (c) Copyright 2010 Xilinx, Inc. All rights reserved.
--
-- This file contains confidential and proprietary information
-- of Xilinx, Inc. and is protected under U.S. and
-- international copyright and other intellectual property
-- laws.
--
-- DISCLAIMER
-- This disclaimer is not a license and does not grant any
-- rights to the materials distributed herewith. Except as
-- otherwise provided in a valid license issued to you by
-- Xilinx, and to the maximum extent permitted by applicable
-- law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
-- WITH ALL FAULTS, AND XILINX HEREBY DISCLAIMS ALL WARRANTIES
-- AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
-- BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
-- INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
-- (2) Xilinx shall not be liable (whether in contract or tort,
-- including negligence, or under any other theory of
-- liability) for any loss or damage of any kind or nature
-- related to, arising under or in connection with these
-- materials, including for any direct, or any indirect,
-- special, incidental, or consequential loss or damage
-- (including loss of data, profits, goodwill, or any type of
-- loss or damage suffered as a result of any action brought
-- by a third party) even if such damage or loss was
-- reasonably foreseeable or Xilinx had been advised of the
-- possibility of the same.
--
-- CRITICAL APPLICATIONS
-- Xilinx products are not designed or intended to be fail-
-- safe, or for use in any application requiring fail-safe
-- performance, such as life-support or safety devices or
-- systems, Class III medical devices, nuclear facilities,
-- applications related to the deployment of airbags, or any
-- other applications that could lead to death, personal
-- injury, or severe property or environmental damage
-- (individually and collectively, "Critical
-- Applications"). Customer assumes the sole risk and
-- liability of any use of Xilinx products in Critical
-- Applications, subject only to applicable laws and
-- regulations governing limitations on product liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
-- PART OF THIS FILE AT ALL TIMES.
--
--
--         $Id: version_reg.vhd,v 1.1 2010/10/12 18:33:54 walker Exp $
--         $URL: http://psgsvn:8690/repos/mvi/trunk/VideoAnalytics/Generation_A/Project_Docs/Xilinx_Header.vhd $
--         $Author: walker $
--         $Revision: 1.1 $
--         $Date: 2010/10/12 18:33:54 $
--
--         Note:  For the above information to be update in SVN
--                 you need to add SVN property svn: keywords Id URL Author Revision Date
-- Description -
--
--     This module implements a simple version number mechanism
--   to be used with all MVI video analytics IP.  This module
--   provides version number capabilities for both the standard
--   LogiCORE version as well as the EDK version methods.  Please
--   see the "version_register.docx" specification for details
--   on how to use this module correctly.
--******************************************************************
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.numeric_std.all;

package version_reg_pkg is
  
component version_reg is
  
  generic (
    VERSION_MAJOR     : std_logic_vector (3 downto 0) := X"1";   -- Major Version number
    VERSION_MINOR     : std_logic_vector (7 downto 0) := X"00";  -- Minor Version Number
    VERSION_REVISION  : std_logic_vector (3 downto 0) := X"a";   -- Version Revision character (EDK)
    REVISION_NUMBER   : string := "Build Number: 0000"           -- Internal build number
    );
  port (
    version : out std_logic_vector (31 downto 0)  -- Version number of IP output
    );
end component;

end version_reg_pkg;


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.numeric_std.all;


entity version_reg is
  
  generic (
    VERSION_MAJOR     : std_logic_vector (3 downto 0) := X"1";   -- Major Version number
    VERSION_MINOR     : std_logic_vector (7 downto 0) := X"00";  -- Minor Version Number
    VERSION_REVISION  : std_logic_vector (3 downto 0) := X"a";   -- Version Revision character (EDK)
    REVISION_NUMBER   : string := "Build Number: 0000"           -- Internal build number
    );
  port (
    version : out std_logic_vector (31 downto 0)  -- Version number of IP output
    );
end version_reg;
  architecture RTL of version_reg is

      -- Helper function
function string2int(buf: string) 
  return integer is
  variable result : integer := 0;
  begin
    for i in 1 to buf'length loop
            case buf(i) is
                    when '0' => result := result*10;
                    when '1' => result := result*10 + 1;
                    when '2' => result := result*10 + 2;
                    when '3' => result := result*10 + 3;
                    when '4' => result := result*10 + 4;
                    when '5' => result := result*10 + 5;
                    when '6' => result := result*10 + 6;
                    when '7' => result := result*10 + 7;
                    when '8' => result := result*10 + 8;
                    when '9' => result := result*10 + 9;
                    when others => null;
            end case;
    end loop;
    return result;
  end;  

  signal rev_num : integer := string2int(REVISION_NUMBER);


  begin  -- RTL

    version(31 downto 16) <= (VERSION_MAJOR & VERSION_MINOR & VERSION_REVISION);
    version(15 downto 0)  <= CONV_STD_LOGIC_VECTOR(rev_num,16);

  end RTL;

