--*******************************************************************
-- (c) Copyright 2010 Xilinx, Inc. All rights reserved.
--
-- This file contains confidential and proprietary information
-- of Xilinx, Inc. and is protected under U.S. and
-- international copyright and other intellectual property
-- laws.
--
-- DISCLAIMER
-- This disclaimer is not a license and does not grant any
-- rights to the materials distributed herewith. Except as
-- otherwise provided in a valid license issued to you by
-- Xilinx, and to the maximum extent permitted by applicable
-- law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
-- WITH ALL FAULTS, AND XILINX HEREBY DISCLAIMS ALL WARRANTIES
-- AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
-- BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
-- INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
-- (2) Xilinx shall not be liable (whether in contract or tort,
-- including negligence, or under any other theory of
-- liability) for any loss or damage of any kind or nature
-- related to, arising under or in connection with these
-- materials, including for any direct, or any indirect,
-- special, incidental, or consequential loss or damage
-- (including loss of data, profits, goodwill, or any type of
-- loss or damage suffered as a result of any action brought
-- by a third party) even if such damage or loss was
-- reasonably foreseeable or Xilinx had been advised of the
-- possibility of the same.
--
-- CRITICAL APPLICATIONS
-- Xilinx products are not designed or intended to be fail-
-- safe, or for use in any application requiring fail-safe
-- performance, such as life-support or safety devices or
-- systems, Class III medical devices, nuclear facilities,
-- applications related to the deployment of airbags, or any
-- other applications that could lead to death, personal
-- injury, or severe property or environmental damage
-- (individually and collectively, "Critical
-- Applications"). Customer assumes the sole risk and
-- liability of any use of Xilinx products in Critical
-- Applications, subject only to applicable laws and
-- regulations governing limitations on product liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
-- PART OF THIS FILE AT ALL TIMES.
--
--  All rights reserved.
--
--******************************************************************
--
-- Filename - osd_counters.vhd
-- Author -   Chris Martin, Xilinx
-- Creation - February 15, 2008
--
-- Description - Horizontal and Vertical Pixel Counters
-- This module counts the active H/W pixels.  It reads fsync, hblank
-- and vblank. Outputs are the H/W counts (12 bits) and a 2 bit 
-- component counter (counts to 0, 1 or 2).  A line_end output is
-- also asserted to denote the end of a line. A read_for_data output
-- is asserted high when the layer should request data.
-- 
--*******************************************************************
-- $Id: osd_counters.vhd 8788 2010-09-01 19:49:10Z chrisma $
-- $URL: http://psgsvn:8690/repos/mvi/trunk/VideoAnalytics/dev/OSD/hdl/osd_counters.vhd $
-- $Author: chrisma $
-- $Revision: 8788 $
-- $Date: 2010-09-01 14:49:10 -0500 (Wed, 01 Sep 2010) $
--*******************************************************************


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_signed.all;
use ieee.std_logic_arith.CONV_STD_LOGIC_VECTOR;

Library UNISIM;
use UNISIM.vcomponents.all;

Library UNIMACRO;
use UNIMACRO.vcomponents.all;

entity vfbc2axi is
        generic
        (
          C_M_AXIS_S2MM_TDATA_WIDTH : integer := 32;
          C_S_AXIS_MM2S_TDATA_WIDTH : integer := 32
        );
        port
        (
          -- Master AXI Streaming for write side
          m_axis_aresetn        : in  std_logic := '1';
          --m_axis_aclk           : in  std_logic := '0';
          m_axis_tdata          : out std_logic_vector(C_M_AXIS_S2MM_TDATA_WIDTH-1 downto 0);
          m_axis_tstrb          : out std_logic_vector((C_M_AXIS_S2MM_TDATA_WIDTH/8)-1 downto 0);
          m_axis_tvalid         : out std_logic;
          m_axis_tready         : in  std_logic := '0';
          m_axis_tlast          : out std_logic;

          -- write xsvi Interface (connected to Master AXI Streaming)
			 fsync_in              : in std_logic := '0';
          vfbc_wd_clk             : in std_logic := '0';
          vfbc_wd_write          : in std_logic := '0';
          vfbc_wd_data            : in std_logic_vector(C_M_AXIS_S2MM_TDATA_WIDTH-1 downto 0) := (others => '0');
			 vfbc_wd_full						  : out std_logic;
--			 axis_data_count       : out std_logic_vector(10 downto 0);
			 fsync_out             : out std_logic;

          vfbc_rd_clk           : in std_logic := '0';
          vfbc_rd_read          : in std_logic := '0';
          vfbc_rd_data          : out std_logic_vector(C_S_AXIS_MM2S_TDATA_WIDTH-1 downto 0);
          vfbc_rd_almost_empty  : out std_logic;
			 
          s_axis_tdata  : in std_logic_vector(C_S_AXIS_MM2S_TDATA_WIDTH-1 downto 0) := (others => '0');
          s_axis_tvalid : in std_logic := '0';
          s_axis_tready : out std_logic;
          buffer_almost_empty  : in std_logic := '0'

        );
end vfbc2axi;

architecture rtl of vfbc2axi is

signal active_video_d : std_logic;
signal last : std_logic;
signal s_axis_tdata_d : std_logic_vector(C_M_AXIS_S2MM_TDATA_WIDTH-1 downto 0);
signal reset_hold, reset_hold_2, reset_hold_2_p : std_logic;
signal EMPTY : std_logic;
signal do, di : std_logic_vector(C_M_AXIS_S2MM_TDATA_WIDTH downto 0);
signal rdcount, wrcount : std_logic_vector(9 downto 0);

  function GetFifoSize (
    d_width : in integer
    ) return string is
  begin
    if(d_width = 16) then
	   return "18Kb";
    else
      return "36Kb";
    end if;
  end;

		  
begin

    s_axis_tready <= vfbc_rd_read;
	 vfbc_rd_almost_empty <= buffer_almost_empty;
	 
	 process(vfbc_rd_clk)
	 begin
		if rising_edge(vfbc_rd_clk) then
--		  if (vfbc_rd_read = '1' and s_axis_tvalid = '1') then
		    vfbc_rd_data <= s_axis_tdata;
--		  end if;
		end if;
	 end process;
	 
    process(vfbc_wd_clk) 
    begin
      if rising_edge(vfbc_wd_clk) then
        if(m_axis_aresetn = '0') then
          active_video_d   <= '0';
			 reset_hold <= '0';
			 reset_hold_2 <= '0';
        else
          active_video_d    <= vfbc_wd_write;-- and reset_hold_2;
			 fsync_out <= fsync_in and reset_hold;
			 if (fsync_in = '1') then
				reset_hold_2 <= reset_hold;
				reset_hold <= '1';
			 end if;
        end if;
        s_axis_tdata_d     <= vfbc_wd_data;
      end if;
    end process;
  
        last <= active_video_d and not vfbc_wd_write;
		  m_axis_tstrb <= (others=>'1');
		  m_axis_tlast <= do(C_M_AXIS_S2MM_TDATA_WIDTH);
		  m_axis_tdata <= do(C_M_AXIS_S2MM_TDATA_WIDTH-1 downto 0);
		  di <= last & s_axis_tdata_d;

--        coregen_fifo : xsvi2axi_fifo_16_1024
--                port map (
--                        s_aclk => vfbc_wd_clk,
--                        s_aresetn => reset_hold_2,
--                        s_axis_tvalid => active_video_d,
--                        s_axis_tready => vfbc_wd_full,
--                        s_axis_tdata => s_axis_tdata_d,
--                        s_axis_tlast => last,
--                        m_axis_tvalid => m_axis_tvalid,
--                        m_axis_tready => m_axis_tready,
--                        m_axis_tdata => m_axis_tdata,
--                        m_axis_tlast => m_axis_tlast,
--								axis_data_count => axis_data_count);

reset_hold_2_p <= not reset_hold_2;
m_axis_tvalid <= not EMPTY;

FIFO_DUALCLOCK_MACRO_inst : FIFO_DUALCLOCK_MACRO
   generic map (
      DEVICE => "7SERIES",            -- Target Device: "VIRTEX5, "VIRTEX6" 
--      ALMOST_FULL_OFFSET => X"0080",  -- Sets almost full threshold
--      ALMOST_EMPTY_OFFSET => X"0080", -- Sets the almost empty threshold
      DATA_WIDTH => C_M_AXIS_S2MM_TDATA_WIDTH+1, -- Valid values are 1-72 (37-72 only valid when FIFO_SIZE="36Kb")
      FIFO_SIZE => GetFifoSize(C_M_AXIS_S2MM_TDATA_WIDTH),-- Target BRAM, "18Kb" or "36Kb" 
		FIRST_WORD_FALL_THROUGH => TRUE)
   port map (
      ALMOSTEMPTY => open,          -- 1-bit output almost empty
      ALMOSTFULL => open,           -- 1-bit output almost full
      DO => do,                     -- Output data, width defined by DATA_WIDTH parameter
      EMPTY => EMPTY,               -- 1-bit output empty
      FULL => vfbc_wd_full,         -- 1-bit output full
      RDCOUNT => rdcount,           -- Output read count, width determined by FIFO depth
      RDERR => open,                -- 1-bit output read error
      WRCOUNT => wrcount,           -- Output write count, width determined by FIFO depth
      WRERR => open,                -- 1-bit output write error
      DI => di,                     -- Input data, width defined by DATA_WIDTH parameter
      RDCLK => vfbc_wd_clk,           -- 1-bit input clock
      RDEN => m_axis_tready,        -- 1-bit input read enable
      RST => reset_hold_2_p,        -- 1-bit input reset
      WRCLK => vfbc_wd_clk,           -- 1-bit input clock
      WREN => active_video_d        -- 1-bit input write enable
   );
	
end rtl;
