-------------------------------------------------------------------------------
-- $Id:
-------------------------------------------------------------------------------
-- zynq_addr_switch - entity/architecture pair
-------------------------------------------------------------------------------
--  **  Copyright(C) 2012 Xilinx, Inc. All rights reserved.         
--  **                                                                       
--  **  This text contains proprietary, confidential information of          
--  **  Xilinx, Inc. , is distributed by under license from Xilinx, Inc.,    
--  **  and may be used, copied and/or disclosed only pursuant to the        
--  **  terms of a valid license agreement with Xilinx, Inc.                 
--  **                                                                       
--  **  Unmodified source code is guaranteed to place and route,             
--  **  function and run at speed according to the datasheet                 
--  **  specification. Source code is provided "as-is", with no              
--  **  obligation on the part of Xilinx to provide support.                 
--  **                                                                       
--  **  Xilinx Hotline support of source code IP shall only include          
--  **  standard level Xilinx Hotline support, and will only address         
--  **  issues and questions related to the standard released Netlist        
--  **  version of the core (and thus indirectly, the original core source). 
--  **                                                                       
--  **  The Xilinx Support Hotline does not have access to source            
--  **  code and therefore cannot answer specific questions related          
--  **  to source HDL. The Xilinx Support Hotline will only be able          
--  **  to confirm the problem in the Netlist version of the core.           
--  **                                                                       
--  **  This copyright and support notice must be retained as part           
--  **  of this text at all times.                                           
-------------------------------------------------------------------------------
-- Filename:        zynq_addr_switch.vhd
-- Version:         v1.00a
-- Description:     This example IP shows how to switch AXI addresses aligned
--		    on 16 MB buffer that is used with the application note.
--                  Basically this IP is just wires to realign the AXI address.
--		    Note this is an example and only uses two masters setup only
--                  for application note only.
--
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:   
--                  zynq_addr_switch.vhd
-- Author:      JEL
--
-- History:
--        JEL         08/7/2012
-- ^^^^^^
--          Initial release.
-- ~~~~~~
-------------------------------------------------------------------------------
--- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_cmb" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


library zynq_addr_switch_v1_00_a;



entity zynq_addr_switch is
  port
  (

    M0_AXI_AWADDR            : in  std_logic_vector(31 downto 0); 
    M1_AXI_AWADDR            : in  std_logic_vector(31 downto 0); 

    
    M0_AXI_ARADDR            : in  std_logic_vector(31 downto 0);
    M1_AXI_ARADDR            : in  std_logic_vector(31 downto 0);
  
    S_AXI_AWADDR            : out  std_logic_vector(63 downto 0); 

    
    S_AXI_ARADDR            : out  std_logic_vector(63 downto 0)
    
    
  );
  
end entity zynq_addr_switch;

------------------------------------------------------------------------------
-- Architecture section
------------------------------------------------------------------------------

architecture IMP of zynq_addr_switch is


begin
  S_AXI_AWADDR <= M1_AXI_AWADDR(31 downto 27)&M1_AXI_AWADDR(23 downto 12)&M1_AXI_AWADDR(26 downto 24)&M1_AXI_AWADDR(11 downto 0)&
  		  M0_AXI_AWADDR(31 downto 27)&M0_AXI_AWADDR(23 downto 12)&M0_AXI_AWADDR(26 downto 24)&M0_AXI_AWADDR(11 downto 0);
 

  S_AXI_ARADDR  <= M1_AXI_ARADDR(31 downto 27)&M1_AXI_ARADDR(23 downto 12)&M1_AXI_ARADDR(26 downto 24)&M1_AXI_ARADDR(11 downto 0)&
  		   M0_AXI_ARADDR(31 downto 27)&M0_AXI_ARADDR(23 downto 12)&M0_AXI_ARADDR(26 downto 24)&M0_AXI_ARADDR(11 downto 0);
  		   
  		   
end IMP;
