//----------------------------------------------------------------
//      _____
//     /     \
//    /____   \____
//   / \===\   \==/
//  /___\===\___\/  AVNET
//       \======/
//        \====/
//---------------------------------------------------------------
//
// This design is the property of Avnet.  Publication of this
// design is not authorized without written consent from Avnet.
//
// Please direct any questions to:  technical.support@avnet.com
//
// Disclaimer:
//    Avnet, Inc. makes no warranty for the use of this code or design.
//    This code is provided  "As Is". Avnet, Inc assumes no responsibility for
//    any errors, which may appear in this code, nor does it make a commitment
//    to update the information contained herein. Avnet, Inc specifically
//    disclaims any implied warranties of fitness for a particular purpose.
//                     Copyright(c) 2010 Avnet, Inc.
//                             All rights reserved.
//
//----------------------------------------------------------------
//
// Create Date:         Mar 19, 2010
// Design Name:         IVK OSD Tutorial
// Module Name:         ivk_osd.c
// Project Name:        IVK
// Target Devices:      Spartan-6
// Avnet Boards:        IVK
//
// Tool versions:       ISE 11.4
//
// Description:         IVK OSD Tutorial
//                      - using the Xilinx OSD LogiCore
//
// Dependencies:
//
// Revision:            Mar 19, 2010: 1.00 Initial version
//                      Apr 16, 2010: 1.01 Update OSD code:
//                                         - convert Font to big-endian
//                                         - draw box (ivk_osd_test1)
//                                         - draw text (ivk_osd_test2)
//
//----------------------------------------------------------------

#include "ivk_osd.h"

// Located in: microblaze_0/include/
#include "xbasic_types.h"
#include "xutil.h"
#include "xparameters.h"
#include "xstatus.h"

#include "ivk_osd.h"
#include "ivk_video_resolution.h"

#include "xosd.h"

/*
 * Device related constants. Defined in xparameters.h
 */
#define OSD_DEVICE_ID     XPAR_OSD_0_DEVICE_ID

/*
 * OSD Device related data structures
 */
XOSD Osd;                    /* OSD Device driver instance */
XOSD_Config *OsdCfgPtr;      /* OSD device configuration pointer */

/*
 * Color table definition
 */
u32 ColorData[16] = {
    0x00000000, 0xa0a25f58, 0xa08080ff, 0xa0808010,
    0xa0ef5a51, 0x00000000, 0xa0465289, 0x00000000,
    0xa065ba6b, 0x00000000, 0xa09017c5, 0xa0a9c860,
    0xa0bc3198, 0xa010a5a9, 0xa0808080, 0xa0ada1ab
};

/*
 * Text table definition
 */
char __attribute__ ((aligned (4))) TextData[8][32] = {
    "Xilinx Virtex6 Video", //"String #1",
    "Scaler/MANR/OSD Demo",                       //"String #2",
    "Hello",
    "Xilinx",
    "String #5",
    "String #6",
    "String #7",
    "String #8"
};

/*
 * Font definition
 */
unsigned char __attribute__ ((aligned (4))) Font[128][8] = {
    {0x00, 0x36, 0x7F, 0x7F, 0x3E, 0x1C, 0x08, 0x00}, // NULL
    {0x18, 0x18, 0x18, 0x1F, 0x1F, 0x18, 0x18, 0x18},
    {0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03},
    {0x18, 0x18, 0x18, 0xF8, 0xF8, 0x00, 0x00, 0x00},
    {0x18, 0x18, 0x18, 0xF8, 0xF8, 0x18, 0x18, 0x18},
    {0x00, 0x00, 0x00, 0xF8, 0xF8, 0x18, 0x18, 0x18},
    {0x03, 0x07, 0x0E, 0x1C, 0x38, 0x70, 0xE0, 0xC0},
    {0xC0, 0xE0, 0x70, 0x38, 0x1C, 0x0E, 0x07, 0x03},
    {0x01, 0x03, 0x07, 0x0F, 0x1F, 0x3F, 0x7F, 0xFF},
    {0x00, 0x00, 0x00, 0x00, 0x0F, 0x0F, 0x0F, 0x0F},
    {0x80, 0xC0, 0xE0, 0xF0, 0xF8, 0xFC, 0xFE, 0xFF},
    {0x0F, 0x0F, 0x0F, 0x0F, 0x00, 0x00, 0x00, 0x00},
    {0xF0, 0xF0, 0xF0, 0xF0, 0x00, 0x00, 0x00, 0x00},
    {0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF},
    {0x00, 0x00, 0x00, 0x00, 0xF0, 0xF0, 0xF0, 0xF0},
    {0x00, 0x1C, 0x1C, 0x77, 0x77, 0x08, 0x1C, 0x00},
    {0x00, 0x00, 0x00, 0x1F, 0x1F, 0x18, 0x18, 0x18},
    {0x00, 0x00, 0x00, 0xFF, 0xFF, 0x00, 0x00, 0x00},
    {0x18, 0x18, 0x18, 0xFF, 0xFF, 0x18, 0x18, 0x18},
    {0x00, 0x00, 0x3C, 0x7E, 0x7E, 0x7E, 0x3C, 0x00},
    {0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF},
    {0xC0, 0xC0, 0xC0, 0xC0, 0xC0, 0xC0, 0xC0, 0xC0},
    {0x00, 0x00, 0x00, 0xFF, 0xFF, 0x18, 0x18, 0x18},
    {0x18, 0x18, 0x18, 0xFF, 0xFF, 0x00, 0x00, 0x00},
    {0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0},
    {0x18, 0x18, 0x18, 0x1F, 0x1F, 0x00, 0x00, 0x00},
    {0x78, 0x60, 0x78, 0x60, 0x7E, 0x18, 0x1E, 0x00},
    {0x00, 0x18, 0x3C, 0x7E, 0x18, 0x18, 0x18, 0x00},
    {0x00, 0x18, 0x18, 0x18, 0x7E, 0x3C, 0x18, 0x00},
    {0x00, 0x18, 0x30, 0x7E, 0x30, 0x18, 0x00, 0x00},
    {0x00, 0x18, 0x0C, 0x7E, 0x0C, 0x18, 0x00, 0x00},

    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // Space
    {0x00, 0x18, 0x18, 0x18, 0x18, 0x00, 0x18, 0x00},
    {0x00, 0x66, 0x66, 0x66, 0x00, 0x00, 0x00, 0x00},
    {0x00, 0x66, 0xFF, 0x66, 0x66, 0xFF, 0x66, 0x00},
    {0x18, 0x3E, 0x60, 0x3C, 0x06, 0x7C, 0x18, 0x00},
    {0x00, 0x66, 0x6C, 0x18, 0x30, 0x66, 0x46, 0x00},
    {0x1C, 0x36, 0x1C, 0x38, 0x6F, 0x66, 0x3B, 0x00},
    {0x00, 0x18, 0x18, 0x18, 0x00, 0x00, 0x00, 0x00},
    {0x00, 0x0E, 0x1C, 0x18, 0x18, 0x1C, 0x0E, 0x00},
    {0x00, 0x70, 0x38, 0x18, 0x18, 0x38, 0x70, 0x00},
    {0x00, 0x66, 0x3C, 0xFF, 0x3C, 0x66, 0x00, 0x00},
    {0x00, 0x18, 0x18, 0x7E, 0x18, 0x18, 0x00, 0x00},
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x18, 0x18, 0x30},
    {0x00, 0x00, 0x00, 0x7E, 0x00, 0x00, 0x00, 0x00},
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x18, 0x18, 0x00},
    {0x00, 0x06, 0x0C, 0x18, 0x30, 0x60, 0x40, 0x00},
    {0x00, 0x3C, 0x66, 0x6E, 0x76, 0x66, 0x3C, 0x00},
    {0x00, 0x18, 0x38, 0x18, 0x18, 0x18, 0x7E, 0x00},
    {0x00, 0x3C, 0x66, 0x0C, 0x18, 0x30, 0x7E, 0x00},
    {0x00, 0x7E, 0x0C, 0x18, 0x0C, 0x66, 0x3C, 0x00},
    {0x00, 0x0C, 0x1C, 0x3C, 0x6C, 0x7E, 0x0C, 0x00},
    {0x00, 0x7E, 0x60, 0x7C, 0x06, 0x66, 0x3C, 0x00},
    {0x00, 0x3C, 0x60, 0x7C, 0x66, 0x66, 0x3C, 0x00},
    {0x00, 0x7E, 0x06, 0x0C, 0x18, 0x30, 0x30, 0x00},
    {0x00, 0x3C, 0x66, 0x3C, 0x66, 0x66, 0x3C, 0x00},
    {0x00, 0x3C, 0x66, 0x3E, 0x06, 0x0C, 0x38, 0x00},
    {0x00, 0x00, 0x18, 0x18, 0x00, 0x18, 0x18, 0x00},
    {0x00, 0x00, 0x18, 0x18, 0x00, 0x18, 0x18, 0x30},
    {0x06, 0x0C, 0x18, 0x30, 0x18, 0x0C, 0x06, 0x00},
    {0x00, 0x00, 0x7E, 0x00, 0x00, 0x7E, 0x00, 0x00},
    {0x60, 0x30, 0x18, 0x0C, 0x18, 0x30, 0x60, 0x00},
    {0x00, 0x3C, 0x66, 0x0C, 0x18, 0x00, 0x18, 0x00},

    {0x00, 0x3C, 0x66, 0x6E, 0x6E, 0x60, 0x3E, 0x00}, // @
    {0x00, 0x18, 0x3C, 0x66, 0x66, 0x7E, 0x66, 0x00}, // A
    {0x00, 0x7C, 0x66, 0x7C, 0x66, 0x66, 0x7C, 0x00},
    {0x00, 0x3C, 0x66, 0x60, 0x60, 0x66, 0x3C, 0x00},
    {0x00, 0x78, 0x6C, 0x66, 0x66, 0x6C, 0x78, 0x00},
    {0x00, 0x7E, 0x60, 0x7C, 0x60, 0x60, 0x7E, 0x00},
    {0x00, 0x7E, 0x60, 0x7C, 0x60, 0x60, 0x60, 0x00},
    {0x00, 0x3E, 0x60, 0x60, 0x6E, 0x66, 0x3E, 0x00},
    {0x00, 0x66, 0x66, 0x7E, 0x66, 0x66, 0x66, 0x00},
    {0x00, 0x7E, 0x18, 0x18, 0x18, 0x18, 0x7E, 0x00},
    {0x00, 0x06, 0x06, 0x06, 0x06, 0x66, 0x3C, 0x00},
    {0x00, 0x66, 0x6C, 0x78, 0x78, 0x6C, 0x66, 0x00},
    {0x00, 0x60, 0x60, 0x60, 0x60, 0x60, 0x7E, 0x00},
    {0x00, 0x63, 0x77, 0x7F, 0x6B, 0x63, 0x63, 0x00},
    {0x00, 0x66, 0x76, 0x7E, 0x7E, 0x6E, 0x66, 0x00},
    {0x00, 0x3C, 0x66, 0x66, 0x66, 0x66, 0x3C, 0x00},
    {0x00, 0x7C, 0x66, 0x66, 0x7C, 0x60, 0x60, 0x00},
    {0x00, 0x3C, 0x66, 0x66, 0x66, 0x6C, 0x36, 0x00},
    {0x00, 0x7C, 0x66, 0x66, 0x7C, 0x6C, 0x66, 0x00},
    {0x00, 0x3C, 0x60, 0x3C, 0x06, 0x06, 0x3C, 0x00},
    {0x00, 0x7E, 0x18, 0x18, 0x18, 0x18, 0x18, 0x00},
    {0x00, 0x66, 0x66, 0x66, 0x66, 0x66, 0x7E, 0x00},
    {0x00, 0x66, 0x66, 0x66, 0x66, 0x3C, 0x18, 0x00},
    {0x00, 0x63, 0x63, 0x6B, 0x7F, 0x77, 0x63, 0x00},
    {0x00, 0x66, 0x66, 0x3C, 0x3C, 0x66, 0x66, 0x00},
    {0x00, 0x66, 0x66, 0x3C, 0x18, 0x18, 0x18, 0x00},
    {0x00, 0x7E, 0x0C, 0x18, 0x30, 0x60, 0x7E, 0x00},
    {0x00, 0x1E, 0x18, 0x18, 0x18, 0x18, 0x1E, 0x00},
    {0x00, 0x40, 0x60, 0x30, 0x18, 0x0C, 0x06, 0x00},
    {0x00, 0x78, 0x18, 0x18, 0x18, 0x18, 0x78, 0x00},
    {0x00, 0x08, 0x1C, 0x36, 0x63, 0x00, 0x00, 0x00},
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0x00},

    {0x40, 0x40, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00}, // `
    {0x00, 0x00, 0x3C, 0x06, 0x3E, 0x66, 0x3E, 0x00}, // a
    {0x00, 0x60, 0x60, 0x7C, 0x66, 0x66, 0x7C, 0x00},
    {0x00, 0x00, 0x3C, 0x60, 0x60, 0x60, 0x3C, 0x00},
    {0x00, 0x06, 0x06, 0x3E, 0x66, 0x66, 0x3E, 0x00},
    {0x00, 0x00, 0x3C, 0x66, 0x7E, 0x60, 0x3C, 0x00},
    {0x00, 0x0E, 0x18, 0x3E, 0x18, 0x18, 0x18, 0x00},
    {0x00, 0x00, 0x3E, 0x66, 0x66, 0x3E, 0x06, 0x7C},
    {0x00, 0x60, 0x60, 0x7C, 0x66, 0x66, 0x66, 0x00},
    {0x00, 0x18, 0x00, 0x38, 0x18, 0x18, 0x3C, 0x00},
    {0x00, 0x06, 0x00, 0x06, 0x06, 0x06, 0x06, 0x3C},
    {0x00, 0x60, 0x60, 0x6C, 0x78, 0x6C, 0x66, 0x00},
    {0x00, 0x38, 0x18, 0x18, 0x18, 0x18, 0x3C, 0x00},
    {0x00, 0x00, 0x66, 0x7F, 0x7F, 0x6B, 0x63, 0x00},
    {0x00, 0x00, 0x7C, 0x66, 0x66, 0x66, 0x66, 0x00},
    {0x00, 0x00, 0x3C, 0x66, 0x66, 0x66, 0x3C, 0x00},
    {0x00, 0x00, 0x7C, 0x66, 0x66, 0x7C, 0x60, 0x60},
    {0x00, 0x00, 0x3E, 0x66, 0x66, 0x3E, 0x06, 0x06},
    {0x00, 0x00, 0x7C, 0x66, 0x60, 0x60, 0x60, 0x00},
    {0x00, 0x00, 0x3E, 0x60, 0x3C, 0x06, 0x7C, 0x00},
    {0x00, 0x18, 0x7E, 0x18, 0x18, 0x18, 0x0E, 0x00},
    {0x00, 0x00, 0x66, 0x66, 0x66, 0x66, 0x3E, 0x00},
    {0x00, 0x00, 0x66, 0x66, 0x66, 0x3C, 0x18, 0x00},
    {0x00, 0x00, 0x63, 0x6B, 0x7F, 0x3E, 0x36, 0x00},
    {0x00, 0x00, 0x66, 0x3C, 0x18, 0x3C, 0x66, 0x00},
    {0x00, 0x00, 0x66, 0x66, 0x66, 0x3E, 0x0C, 0x78},
    {0x00, 0x00, 0x7E, 0x0C, 0x18, 0x30, 0x7E, 0x00},

    {0x14, 0x10, 0x10, 0x40, 0x10, 0x10, 0x14, 0x00}, // {
    {0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18},
    {0x50, 0x10, 0x10, 0x04, 0x10, 0x10, 0x50, 0x00}, // }
    {0x11, 0x11, 0x44, 0x44, 0x00, 0x00, 0x00, 0x00}, // ~
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // 127 DEL
};

/*
 * Function prototypes
 */
static int  OsdInit(int DeviceID);
static void OsdConfig(int Width, int Height);
static void OsdDrawBox(int x_start, int y_start, int x_last, int y_last, int color);
static void OsdDrawText(int x_pos, int y_pos, int color, int string_index, int text_size);

Xuint32 ivk_osd_init(Xuint32 resolutionId)
{
   static int bOsdInitialized = 0;
   Xuint32 Width  = ivk_vres_get_width(resolutionId);
   Xuint32 Height = ivk_vres_get_height(resolutionId);

   if ( !bOsdInitialized )
   {
      OsdInit(OSD_DEVICE_ID);
      OsdConfig(Width,Height);
      bOsdInitialized = 1;
   }
}


Xuint32 ivk_osd_test1(void)
{
   OsdDrawBox(
        20, // X start
        20, // Y start
      1260, // X end
       100, // Y end
         1  // color index
         );
}

Xuint32 ivk_osd_test2(void)
{
   OsdDrawText(
       128, // X start
        40, // Y start
         2, // color index
         0, // string index
         4  // text scale
         );
}

Xuint32 ivk_osd_test3(void)
{
   Xuint32 idx = 0x00;
   Xuint32 reg = 0x00000000;
#if 0
   reg = XOSD_ReadReg(Osd.Config.BaseAddress, XOSD_CTL);
   xil_printf( "XOSD_CTL = 0x%08X\n\r", reg );
#endif
#if 1
   for ( idx = 0x0000; idx < 0x0050; idx += 4 )
   {
      if ( !(idx%16) ) xil_printf( "\r\n[0x%04X] = ", idx );
      reg = XOSD_ReadReg(Osd.Config.BaseAddress, idx);
      xil_printf( "0x%08X ", reg );
   }
   for ( idx = 0xA0; idx < 0xB0; idx += 4 )
   {
      if ( !(idx%16) ) xil_printf( "\r\n[0x%04X] = ", idx );
      reg = XOSD_ReadReg(Osd.Config.BaseAddress, idx);
      xil_printf( "0x%08X ", reg );
   }
   idx = 0x100;
      reg = XOSD_ReadReg(Osd.Config.BaseAddress, idx);
      xil_printf( "\r\n[0x%04X] = 0x%08X ", idx, reg );
   idx = 0x21C;
      reg = XOSD_ReadReg(Osd.Config.BaseAddress, idx);
      xil_printf( "\r\n[0x%04X] = 0x%08X ", idx, reg );
   idx = 0x220;
      reg = XOSD_ReadReg(Osd.Config.BaseAddress, idx);
      xil_printf( "\r\n[0x%04X] = 0x%08X ", idx, reg );
   idx = 0x228;
      reg = XOSD_ReadReg(Osd.Config.BaseAddress, idx);
      xil_printf( "\r\n[0x%04X] = 0x%08X ", idx, reg );
   idx = 0x22C;
      reg = XOSD_ReadReg(Osd.Config.BaseAddress, idx);
      xil_printf( "\r\n[0x%04X] = 0x%08X ", idx, reg );
   xil_printf( "\r\n" );
#endif
}

Xuint32 ivk_osd_test4(void)
{
}

/*****************************************************************************/
/**
*
* This function initializes the OSD device and its driver instance.
*
* @param    DeviceID is the device ID of the OSD device.
*
* @return   0 if the initialization is successful; 1 otherwise.
*
* @note     None.
*
******************************************************************************/
int OsdInit(int DeviceID)
{
    int Status;

   xil_printf("OsdInit(%d) Start\r\n", DeviceID );

    /* Initialize the OSD instance */
   xil_printf("-- Initialize the OSD instance ...\r\n" );

    OsdCfgPtr = XOSD_LookupConfig(DeviceID);
    Status = XOSD_CfgInitialize(&Osd, OsdCfgPtr, OsdCfgPtr->BaseAddress);
    if (Status != XST_SUCCESS) {
        return 1;
    }

    /* Reset the devices */
   xil_printf("-- Reset the devices ...\r\n" );

    XOSD_Reset(&Osd);


    /* Set Vertical and Horizontal Blank Polarity */
   xil_printf("-- Set Vertical and Horizontal Blank Polarity ...\r\n" );

    XOSD_SetBlankPolarity(&Osd,TRUE,TRUE);

//    /* Enable the OSD device and tell it to pick up the register changes */
//   xil_printf("-- Enable the OSD device and tell it to pick up the register changes ...\r\n" );
//
//    XOSD_Enable(&Osd);
//    XOSD_RegUpdateEnable(&Osd);

    /* Tell the OSD to pick up the register changes */
   xil_printf("-- Tell the OSD to pick up the register changes ...\r\n" );

    XOSD_RegUpdateEnable(&Osd);


#if 0 // NO , don't do this, nothing works
    // BUG FIX :
    //  XOSD_CTL bits are reversed in hardware
    {
      Xuint32 reg1;
      Xuint32 reg2;
      int i;

      reg1 = XOSD_ReadReg(Osd.Config.BaseAddress, XOSD_CTL);

      // TODO : add automatic detection of bit reverse issue (if possible)
      {
//#ifdef DEBUG
         xil_printf( "Applying bit reverse fix to XOSD_CTL register:\r\n" );
//#endif
//#ifdef DEBUG
         xil_printf( "-- Before bit reverse bug fix : XOSD_CTL = 0x%08X\r\n", reg1 );
//#endif
         reg2 = 0x00000000;
         for ( i = 0; i < 32; i++ )
         {
            if ( reg1 & (1<<i) )
            {
               reg2 |= (1<<(31-i));
            }
         }
//#ifdef DEBUG
         xil_printf( "-- After  bit revsere bug fix : XOSD_CTL = 0x%08X\r\n", reg2 );
//#endif

         XOSD_WriteReg(Osd.Config.BaseAddress, XOSD_CTL, reg2 );
      }
   }
#endif


   xil_printf("OsdInit Done\r\n" );
    return 0;
}

/*****************************************************************************/
/**
*
* This function does the general configuration on an OSD device. The
* configuration includes:
*
*   - Screen Size
*   - Background Color
*   - Layer 0 setup: Alpha, Priority, Dimension and enabling
*   - Layer 1 setup: Alpha, Priority, Dimension and enabling
*   - Loading Color/Font/Text configuration
*
* @param    None.
*
* @return   None.
*
* @note     None.
*
******************************************************************************/
static void OsdConfig( int Width, int Height )
{
   /* Background color definition */

   u8 Red = 0x80; //0xFF;
   u8 Blue = 0x80; //0;
   u8 Green = 0x10; //0;

   /* Layer 0/1 property definition */

   int Layer0AlphaValue = 0xFF;
   int Layer0Priority = XOSD_LAYER_PRIORITY_0;
   int Layer0GlobalAlphaEnable = 1;

   int Layer1AlphaValue = 0xFF;
   int Layer1Priority = XOSD_LAYER_PRIORITY_1;
   int Layer1GlobalAlphaEnable = 1;

   int Layer2AlphaValue = 0xFF;
   int Layer2Priority = XOSD_LAYER_PRIORITY_2;
   int Layer2GlobalAlphaEnable = 1;

   int Layer3AlphaValue = 0xFF;
   int Layer3Priority = XOSD_LAYER_PRIORITY_3;
   int Layer3GlobalAlphaEnable = 1;

   int Layer4AlphaValue = 0xFF;
   int Layer4Priority = XOSD_LAYER_PRIORITY_4;
   int Layer4GlobalAlphaEnable = 1;

   int Layer5AlphaValue = 0xFF;
   int Layer5Priority = XOSD_LAYER_PRIORITY_5;
   int Layer5GlobalAlphaEnable = 1;

   int Layer6AlphaValue = 0xFF;
   int Layer6Priority = XOSD_LAYER_PRIORITY_6;
   int Layer6GlobalAlphaEnable = 1;

   int Layer7AlphaValue = 0xFF;
   int Layer7Priority = XOSD_LAYER_PRIORITY_7;
   int Layer7GlobalAlphaEnable = 1;

   xil_printf("OsdConfig(%d,%d) Start\r\n", Width, Height );

   /* Set screen size */
   xil_printf("-- Set screen size ...\r\n" );

   XOSD_SetScreenSize(&Osd, Width, Height);

   /* Set Background color */
   xil_printf("-- Set Background color ...\r\n" );

   XOSD_SetBackgroundColor(&Osd, Red, Blue, Green);

   /* Set up Layer 0's Alpha, Priority, Dimension and enable it */
   xil_printf("-- Set up Layer 0's Alpha, Priority, Dimension and enable it ...\r\n" );
   XOSD_SetLayerAlpha(&Osd, 0, Layer0GlobalAlphaEnable, Layer0AlphaValue);
   XOSD_SetLayerPriority(&Osd, 0, Layer2Priority);
   XOSD_SetLayerDimension(&Osd, 0, 0, 0, Width, Height);
   XOSD_EnableLayer(&Osd, 0);

   XOSD_SetLayerAlpha(&Osd, 1, Layer1GlobalAlphaEnable, Layer1AlphaValue);
   XOSD_SetLayerPriority(&Osd, 1, Layer1Priority);
   XOSD_SetLayerDimension(&Osd, 1, 0, 0, Width, Height);
   XOSD_EnableLayer(&Osd, 1);

   XOSD_SetLayerAlpha(&Osd, 2, Layer2GlobalAlphaEnable, Layer2AlphaValue);
   XOSD_SetLayerPriority(&Osd, 2, Layer0Priority);
   XOSD_SetLayerDimension(&Osd, 2, 0, 0, Width, Height);
   XOSD_EnableLayer(&Osd, 2);


    /* Enable the OSD device and tell it to pick up the register changes */
   xil_printf("-- Enable the OSD device ...\r\n" );

    XOSD_Enable(&Osd);


   xil_printf("OsdConfig Done\r\n" );
   return;
}

/*****************************************************************************/
/**
*
* This function draws a box using the OSD device
*
* @param    None.
*
* @return   None.
*
* @note     None.
*
******************************************************************************/
static void OsdDrawBox(int x_start, int y_start, int x_last, int y_last, int color)
{
   xil_printf("OsdDrawBox Start\r\n" );

   /* Instruction buffer */
   u32 Instruction[XOSD_INS_SIZE];

   /* Instruction property definition */
   xil_printf("-- Instruction property definition ...\r\n" );

   u16 ObjType = XOSD_INS_OPCODE_BOX; /* A box */
   u8  ObjSize = 2;                   /* Box boarder width */
   u16 XStart = 20;                   /* Horizontal start pixel of the box */
   u16 YStart = 20;                   /* Vertical start line of the box */
   u16 XEnd = 1280-20;                /* Horizontal end pixel of the box */
   u16 YEnd =  100;                   /* Vertical end line of the box */
   u8 TextIndex = 0;                  /* Ignored in box instruction case */
   u8 ColorIndex = color;             /* Color used to draw the box */

   /* Create a box command instruction */
   xil_printf("-- Create a box command instruction ...\r\n" );

   XOSD_CreateInstruction(&Osd, Instruction, 1,
                           ObjType, ObjSize,
                           XStart, YStart, XEnd, YEnd,
                           TextIndex, ColorIndex);

   /* Load the instruction to draw the box in the OSD output */
   xil_printf("-- Load the instruction to draw the box in the OSD output ...\r\n" );

   XOSD_LoadInstructionList(&Osd, 1, 0, Instruction, 1);

   xil_printf("OsdDrawBox Done\r\n" );
   return;
}


/*****************************************************************************/
/**
*
* This function draws text using the OSD device
*
* @param    None.
*
* @return   None.
*
* @note     None.
*
******************************************************************************/
static void OsdDrawText(int x_pos, int y_pos, int color, int string_index, int text_size)
{
   xil_printf("OsdDrawText Start\r\n" );

   /* Instruction buffer */
   u32 Instruction[XOSD_INS_SIZE];


   u16 ObjType = XOSD_INS_OPCODE_TXT; /* A text string */
   u8  ObjSize = (text_size<<4);      /* Text Scale factor (1x, 2x, 4x, 8x) */
   u16 XStart = x_pos;                /* Horizontal start pixel of the text */
   u16 YStart = y_pos;                /* Vertical start line of the text */
   u16 XEnd =   x_pos;                /* Horizontal end pixel of the text (not used) */
   u16 YEnd =   y_pos;                /* Vertical end line of the text (must be same as YStart) */
   u8 TextIndex = 0;                  /* Index of Text String */
   u8 ColorIndex = color;             /* Color used to draw text */

   /* Create a text command instruction */
   xil_printf("-- Create a box command instruction ...\r\n" );

   XOSD_CreateInstruction(&Osd, Instruction, 1,
                           ObjType, ObjSize,
                           XStart, YStart, XEnd, YEnd,
                           TextIndex, ColorIndex);

   /* Load the instruction to draw the box in the OSD output */
   xil_printf("-- Load the instruction to draw the box in the OSD output ...\r\n" );

   XOSD_LoadInstructionList(&Osd, 1, 0, Instruction, 1);

   xil_printf("OsdDrawBox Done\r\n" );
   return;
}

