/* $Id: */
/******************************************************************************
* (c) Copyright 2009 Xilinx, Inc. All rights reserved.
*
* This file contains confidential and proprietary information
* of Xilinx, Inc. and is protected under U.S. and
* international copyright and other intellectual property
* laws.
*
* DISCLAIMER
* This disclaimer is not a license and does not grant any
* rights to the materials distributed herewith. Except as
* otherwise provided in a valid license issued to you by
* Xilinx, and to the maximum extent permitted by applicable
* law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
* WITH ALL FAULTS, AND XILINX HEREBY DISCLAIMS ALL WARRANTIES
* AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
* BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
* INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
* (2) Xilinx shall not be liable (whether in contract or tort,
* including negligence, or under any other theory of
* liability) for any loss or damage of any kind or nature
* related to, arising under or in connection with these
* materials, including for any direct, or any indirect,
* special, incidental, or consequential loss or damage
* (including loss of data, profits, goodwill, or any type of
* loss or damage suffered as a result of any action brought
* by a third party) even if such damage or loss was
* reasonably foreseeable or Xilinx had been advised of the
* possibility of the same.
*
* CRITICAL APPLICATIONS
* Xilinx products are not designed or intended to be fail-
* safe, or for use in any application requiring fail-safe
* performance, such as life-support or safety devices or
* systems, Class III medical devices, nuclear facilities,
* applications related to the deployment of airbags, or any
* other applications that could lead to death, personal
* injury, or severe property or environmental damage
* (individually and collectively, "Critical
* Applications"). Customer assumes the sole risk and
* liability of any use of Xilinx products in Critical
* Applications, subject only to applicable laws and
* regulations governing limitations on product liability.
*
* THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
* PART OF THIS FILE AT ALL TIMES.
*
* All rights reserved.
*
******************************************************************************/
/*****************************************************************************/
/**
*
* @file xtimebase_hw.h
*
* This header file contains identifiers and register-level driver functions (or
* macros) that can be used to access the Xilinx MVI Video TimeBase device.
*
* For more information about the operation of this device, see the hardware
* specification and documentation in the higher level driver xtimebase.h source
* code file.
*
* <pre>
* MODIFICATION HISTORY:
*
* Ver	Who	Date		Changes
* -----	----	--------	-----------------------------------------------
* 1.00a	xd	08/05/08	First release
* 1.01a	xd	07/23/10	Added GIER; Added more h/w generic info into
*				xparameters.h; Feed callbacks with pending
*				interrupt info. Added Doxygen & Version support
* </pre>
*
******************************************************************************/

#ifndef XTIMEBASE_HW_H		  /* prevent circular inclusions */
#define XTIMEBASE_HW_H		  /* by using protection macros */

#ifdef __cplusplus
extern "C" {
#endif

/***************************** Include Files *********************************/

#include "xil_io.h"

/************************** Constant Definitions *****************************/

/** @name Device Register Offsets
 *  @{
 */
#define XTB_CTL		0x000	/**< Control */

#define XTB_GH0		0x004	/**< Generator Horizontal Register 0 */
#define XTB_GH1		0x008	/**< Generator Horizontal Register 1 */
#define XTB_GH2		0x00C	/**< Generator Horizontal Register 2 */
#define XTB_GV0		0x010	/**< Generator Vertical Register 0 */
#define XTB_GV1		0x014	/**< Generator Vertical Register 1 */
#define XTB_GV2		0x018	/**< Generator Vertical Register 2 */
#define XTB_GV3		0x01C	/**< Generator Vertical Register 3 */
#define XTB_GV4		0x020	/**< Generator Vertical Register 4 */
#define XTB_GV5		0x024	/**< Generator Vertical Register 5 */

#define XTB_DS		0x028	/**< Detector Status Register */

#define XTB_DH0		0x02C	/**< Detector Horizontal Register 0 */
#define XTB_DH1		0x030	/**< Detector Horizontal Register 1 */
#define XTB_DH2		0x034	/**< Detector Horizontal Register 2 */
#define XTB_DV0		0x038	/**< Detector Vertical Register 0 */
#define XTB_DV1		0x03C	/**< Detector Vertical Register 1 */
#define XTB_DV2		0x040	/**< Detector Vertical Register 2 */
#define XTB_DV3		0x044	/**< Detector Vertical Register 3 */
#define XTB_DV4		0x048	/**< Detector Vertical Register 4 */
#define XTB_DV5		0x04C	/**< Detector Vertical Register 5 */

#define XTB_FS00	0x050	/**< Frame Sync 00 Config Register */
#define XTB_FS01	0x054	/**< Frame Sync 01 Config Register */
#define XTB_FS02	0x058	/**< Frame Sync 02 Config Register */
#define XTB_FS03	0x05C	/**< Frame Sync 03 Config Register */
#define XTB_FS04	0x060	/**< Frame Sync 04 Config Register */
#define XTB_FS05	0x064	/**< Frame Sync 05 Config Register */
#define XTB_FS06	0x068	/**< Frame Sync 06 Config Register */
#define XTB_FS07	0x06C	/**< Frame Sync 07 Config Register */
#define XTB_FS08	0x070	/**< Frame Sync 08 Config Register */
#define XTB_FS09	0x074	/**< Frame Sync 09 Config Register */
#define XTB_FS10	0x078	/**< Frame Sync 10 Config Register */
#define XTB_FS11	0x07C	/**< Frame Sync 11 Config Register */
#define XTB_FS12	0x080	/**< Frame Sync 12 Config Register */
#define XTB_FS13	0x084	/**< Frame Sync 13 Config Register */
#define XTB_FS14	0x088	/**< Frame Sync 14 Config Register */
#define XTB_FS15	0x08C	/**< Frame Sync 15 Config Register */

#define XTB_GGD		0x090	/**< Generator Global Delay register */

#define XTB_VER		0x0F0	/**< Version Register */
#define XTB_RESET	0x100	/**< Reset Register */
#define XTB_GIER	0x21C	/**< Global Interrupt Enable Register */
#define XTB_ISR		0x220	/**< Interrupt Status Register */
#define XTB_IER		0x228	/**< Interrupt Enable Register */
/*@}*/

/** @name Control Register bit definition
 *  @{
 */
#define XTB_CTL_ACP_MASK	0x04000000 /**< Active Chroma Output Polarity*/
#define XTB_CTL_AVP_MASK	0x02000000 /**< Active Video Output Polarity */
#define XTB_CTL_FIP_MASK	0x01000000 /**< Field ID Output Polarity */
#define XTB_CTL_VBP_MASK	0x00800000 /**< Vertical Blank Output Polarity
					     */
#define XTB_CTL_VSP_MASK	0x00400000 /**< Vertical Sync Output Polarity
					     */
#define XTB_CTL_HBP_MASK	0x00200000 /**< Horizontal Blank Output
					     *  Polarity */
#define XTB_CTL_HSP_MASK	0x00100000 /**< Horizontal Sync Output Polarity
					     */
#define XTB_CTL_ALLP_MASK	0x07F00000 /**< Bit mask for all polarity bits
					     */
#define XTB_CTL_VCSS_MASK	0x00040000 /**< Start of Active Chroma Register
					     *  Source Select */
#define XTB_CTL_VASS_MASK	0x00020000 /**< Vertical Active Video Start
					     *  Register Source Select */
#define XTB_CTL_VBSS_MASK	0x00010000 /**< Vertical Back Porch Start
					     *  Register Source Select */
#define XTB_CTL_VSSS_MASK	0x00008000 /**< Vertical Sync Start Register
					     *  Source Select */
#define XTB_CTL_VFSS_MASK	0x00004000 /**< Vertical Front Porch Start
					     *  Register Source Select */
#define XTB_CTL_VTSS_MASK	0x00002000 /**< Vertical Total Register Source
					     *  Select */
#define XTB_CTL_HASS_MASK	0x00001000 /**< Horizontal Active Video Start
					     *  Register Source Select */
#define XTB_CTL_HBSS_MASK	0x00000800 /**< Horizontal Back Porch Start
					     *  Register Source Select */
#define XTB_CTL_HSSS_MASK	0x00000400 /**< Horizontal Sync Start Register
					     *  Source Select */
#define XTB_CTL_HFSS_MASK	0x00000200 /**< Horizontal Front Porch Start
					     *  Register Source Select */
#define XTB_CTL_HTSS_MASK	0x00000100 /**< Horizontal Total Register
					     *  Source Select */
#define XTB_CTL_ALLSS_MASK	0x0007FF00 /**< Bit mask for all source select
					     */
#define XTB_CTL_GACS_MASK	0x00000010 /**< Generator Active Chroma Skip */
#define XTB_CTL_LP_MASK		0x00000008 /**< Lock Polarity */
#define XTB_CTL_DE_MASK		0x00000002 /**< Timebase Detector Enable */
#define XTB_CTL_GE_MASK		0x00000001 /**< Timebase Generator Enable */
/*@}*/

/** @name TimeBase Generator Horizontal 0
 *  @{
 */
#define XTB_GH0_FPSTART_MASK	0x0FFF0000 /**< Horizontal Front Porch Start
					     *  Cycle Count */
#define XTB_GH0_FPSTART_SHIFT	16	   /**< Bit shift for Horizontal
					     *  Front Porch Start Cycle
					     *  Count */
#define XTB_GH0_TOTAL_MASK	0x00000FFF /**< Total clock cycles per Line
					     */
/*@}*/

/** @name TimeBase Generator Horizontal 1
 *  @{
 */
#define XTB_GH1_BPSTART_MASK   0x0FFF0000 /**< Horizontal Back Porch Start
					    *  Cycle Count */
#define XTB_GH1_BPSTART_SHIFT  16	  /**< Bit shift for Horizontal Back
					    *  Porch Start Cycle Count */
#define XTB_GH1_SYNCSTART_MASK 0x00000FFF /**< Horizontal Sync Start Cycle
					    *  Count */
/*@}*/

/** @name TimeBase Generator Horizontal 2
 *  @{
 */
#define XTB_GH2_ACTIVESTART_MASK 0x00000FFF /**< Horizontal Active Video Start
					      *  Cycle Count */
/*@}*/

/** @name TimeBase Generator Vertical 0 (Field 0)
 *  @{
 */
#define XTB_GV0_FPSTART_MASK   0x0FFF0000  /**< Vertical Front Porch Start
					     *  Cycle Count */
#define XTB_GV0_FPSTART_SHIFT  16	   /**< Bit shift for Vertical Front
					     *  Porch Start Cycle Count */
#define XTB_GV0_TOTAL_MASK	0x00000FFF /**< Total lines per Frame */
/*@}*/

/** @name TimeBase Generator Vertical 1 (Field 0)
 *  @{
 */
#define XTB_GV1_BPSTART_MASK   0x0FFF0000  /**< Vertical Back Porch Start
					     *  Cycle Count */
#define XTB_GV1_BPSTART_SHIFT  16	   /**< Bit shift for Vertical Back
					     *  Porch Start Cycle Count */
#define XTB_GV1_SYNCSTART_MASK 0x00000FFF  /**< Vertical Sync Start Cycle
					     *  Count */
/*@}*/

/** @name TimeBase Generator Vertical 2 (Field 0)
 *  @{
 */
#define XTB_GV2_CHROMASTART_MASK  0x0FFF0000 /**< Active Chroma Start Line
					       *  Count */
#define XTB_GV2_CHROMASTART_SHIFT 16	     /**< Bit shift for Active Chroma
					       *  Start Line Count */
#define XTB_GV2_ACTIVESTART_MASK  0x00000FFF /**< Vertical Active Video Start
					       *  Cycle Count */
/*@}*/

/** @name TimeBase Generator Vertical 3 (Field 1)
 *  @{
 */
#define XTB_GV3_FPSTART_MASK	0x0FFF0000    /**< Vertical Front Porch Start
					       *  Cycle Count */
#define XTB_GV3_FPSTART_SHIFT	16	      /**< Bit shift for Vertical Front
					       *  Porch Start Cycle Count */
#define XTB_GV3_TOTAL_MASK	0x00000FFF    /**< Total lines per Frame */
/*@}*/

/** @name TimeBase Generator Vertical 4 (Field 1)
 *  @{
 */
#define XTB_GV4_BPSTART_MASK	0x0FFF0000    /**< Vertical Back Porch Start
						*  Cycle Count */
#define XTB_GV4_BPSTART_SHIFT	16	      /**< Bit shift for Vertical Back
						*  Porch Start Cycle Count */
#define XTB_GV4_SYNCSTART_MASK	0x00000FFF    /**< Vertical Sync Start Cycle
						*  Count */
/*@}*/

/** @name TimeBase Generator Vertical 5 (Field 1)
 *  @{
 */
#define XTB_GV5_CHROMASTART_MASK  0x0FFF0000  /**< Active Chroma Start Line
						*  Count */
#define XTB_GV5_CHROMASTART_SHIFT 16	      /**< Bit shift for Active Chroma
						*  Start Line Count */
#define XTB_GV5_ACTIVESTART_MASK  0x00000FFF  /**< Vertical Active Video Start
						*  Cycle Count */
/*@}*/

/** @name TimeBase Detector Status
 *  @{
 */
#define XTB_DS_AC_POL_MASK	0x04000000 /**< Active Chroma Output Polarity
					     */
#define XTB_DS_AV_POL_MASK	0x02000000 /**< Active Video Output Polarity
					     */
#define XTB_DS_FID_POL_MASK	0x01000000 /**< Field ID Output Polarity */
#define XTB_DS_VBLANK_POL_MASK	0x00800000 /**< Vertical Blank Output
					     *  Polarity */
#define XTB_DS_VSYNC_POL_MASK	0x00400000 /**< Vertical Sync Output Polarity
					     */
#define XTB_DS_HBLANK_POL_MASK	0x00200000 /**< Horizontal Blank Output
					     *  Polarity */
#define XTB_DS_HSYNC_POL_MASK	0x00100000 /**< Horizontal Sync Output
					     *  Polarity */
#define XTB_DS_ACSKIP_MASK	0x00000010 /**< Detector Active Chroma Skip
					     */
/*@}*/

/** @name TimeBase Detector Horizontal 0
 *  @{
 */
#define XTB_DH0_FPSTART_MASK	0x0FFF0000 /**< Detected Horizontal Front
					     *  Porch Start Cycle Count */
#define XTB_DH0_FPSTART_SHIFT	16	   /**< Bit shift for Detected
					     *  Horizontal Front Porch Start
					     *  Cycle Count */
#define XTB_DH0_TOTAL_MASK	0x00000FFF /**< Detected Total clock cycles
					     *  per Line */
/*@}*/

/** @name TimeBase Detector  Horizontal 1
 *  @{
 */
#define XTB_DH1_BPSTART_MASK	0x0FFF0000 /**< Detected Horizontal Back
					     *  Porch Start Cycle Count */
#define XTB_DH1_BPSTART_SHIFT	16	   /**< Bit shift for Detected
					     *  Horizontal Back Porch Start
					     *  Cycle Count */
#define XTB_DH1_SYNCSTART_MASK	0x00000FFF /**< Detected Horizontal Sync
					     *  Start Cycle Count */
/*@}*/

/** @name TimeBase Detector Horizontal 2
 *  @{
 */
#define XTB_DH2_ACTIVESTART_MASK 0x00000FFF /**< Detected Horizontal Active
					      *  Video Start Cycle Count */
/*@}*/

/** @name TimeBase Detector Vertical 0 (Field 0)
 *  @{
 */
#define XTB_DV0_FPSTART_MASK	0x0FFF0000 /**< Detected Vertical Front Porch
					     *  Start Cycle Count */
#define XTB_DV0_FPSTART_SHIFT	16	   /**< Bit shift for Detected
					     *  Vertical Front Porch Start
					     *  Cycle Count */
#define XTB_DV0_TOTAL_MASK	0x00000FFF /**< Detected Total lines per
					     *  Frame */
/*@}*/

/** @name TimeBase Detector Vertical 1 (Field 0)
 *  @{
 */
#define XTB_DV1_BPSTART_MASK	0x0FFF0000 /**< Detected Vertical Back Porch
					     *  Start Cycle Count */
#define XTB_DV1_BPSTART_SHIFT	16	   /**< Bit shift for Detected
					     *  Vertical Back Porch Start
					     *  Cycle Count */
#define XTB_DV1_SYNCSTART_MASK	0x00000FFF /**< Detected Vertical Sync Start
					     *  Cycle Count */
/*@}*/

/** @name TimeBase Detector Vertical 2 (Field 0)
 *  @{
 */
#define XTB_DV2_CHROMASTART_MASK  0x0FFF0000 /**< Detected Active Chroma Start
					       *  Line Count */
#define XTB_DV2_CHROMASTART_SHIFT 16	     /**< Bit shift for Detected Active
					       *  Chroma Start Line Count */
#define XTB_DV2_ACTIVESTART_MASK  0x00000FFF /**< Detected Vertical Active
					       *  Video Start Cycle Count */
/*@}*/

/** @name TimeBase Detector Vertical 3 (Field 1)
 *  @{
 */
#define XTB_DV3_FPSTART_MASK	0x0FFF0000   /**< Detected Vertical Front Porch
					       *  Start Cycle Count */
#define XTB_DV3_FPSTART_SHIFT	16	     /**< Bit shift for Detected
					       *  Vertical Front Porch Start
					       *  Cycle Count */
#define XTB_DV3_TOTAL_MASK	0x00000FFF   /**< Detected Total lines per
					       *  Frame */
/*@}*/

/** @name TimeBase Detector Vertical 4 (Field 1)
 *  @{
 */
#define XTB_DV4_BPSTART_MASK	0x0FFF0000 /**< Detected Vertical Back Porch
					     *  Start Cycle Count */
#define XTB_DV4_BPSTART_SHIFT	16	   /**< Bit shift for Detected
					     *  Vertical Back Porch Start
					     *  Cycle Count */
#define XTB_DV4_SYNCSTART_MASK	0x00000FFF /**< Detected Vertical Sync Start
					     *  Cycle Count */
/*@}*/

/** @name TimeBase Detector Vertical 5 (Field 1)
 *  @{
 */
#define XTB_DV5_CHROMASTART_MASK  0x0FFF0000 /**< Detected Active Chroma Start
					       *  Line Count */
#define XTB_DV5_CHROMASTART_SHIFT 16	     /**< Bit shift for Detected Active
					      *  Chroma Start Line Count */
#define XTB_DV5_ACTIVESTART_MASK  0x00000FFF /**< Detected Vertical Active
					       *  Video Start Cycle Count */
/*@}*/

/** @name Timebase Generator Global Delay
 *  @{
 */
#define XTB_GGD_VDELAY_MASK	0x0FFF0000 /**< Total lines per frame to delay
					     *	generator output */
#define XTB_GGD_VDELAY_SHIFT	16	   /**< Bit shift for the total
					     *	lines */
#define XTB_GGD_HDELAY_MASK	0x00000FFF /**< Total clock cycles per line to
					     *  delay generator output */
/*@}*/


/** @name Timebase Frame Sync 00 --- 15
 *  @{
 */
#define XTB_FSXX_VSTART_MASK	0x0FFF0000 /**< Vertical line count during
					     *  which current Frame Sync is
					     *  active */
#define XTB_FSXX_VSTART_SHIFT	16	   /**< Bit shift for the vertical
					     * line count */
#define XTB_FSXX_HSTART_MASK	0x00000FFF /**< Horizontal cycle count during
					     *  which current Frame Sync is
					     *  active */
/*@}*/

/** @name Reset Register bit definition
 *  @{
 */
#define XTB_RESET_RESET_MASK	0x0000000A /**< Software Reset */
/*@}*/

/** @name Version Register bit definition
 *  @{
 */
#define XTB_VER_MAJOR_MASK	0xF0000000	/**< Major Version*/
#define XTB_VER_MAJOR_SHIFT	28		/**< Major Version Bit Shift*/
#define XTB_VER_MINOR_MASK	0x0FF00000	/**< Minor Version */
#define XTB_VER_MINOR_SHIFT	20		/**< Minor Version Bit Shift*/
#define XTB_VER_REV_MASK	0x000F0000	/**< Revision Version */
#define XTB_VER_REV_SHIFT	16		/**< Revision Bit Shift*/
/*@}*/

/** @name Global Interrupt Enable Register bit definition
 *  @{
 */
#define XTB_GIER_GIE_MASK	0x80000000 /**< Global interrupt enable */
/*@}*/

/** @name Interrupt Status/Enable Register bit definition
 *  @{
 */
#define XTB_IXR_FSYNC15_MASK	0x80000000 /**< Frame Sync Interrupt 15 */
#define XTB_IXR_FSYNC14_MASK	0x40000000 /**< Frame Sync Interrupt 14 */
#define XTB_IXR_FSYNC13_MASK	0x20000000 /**< Frame Sync Interrupt 13 */
#define XTB_IXR_FSYNC12_MASK	0x10000000 /**< Frame Sync Interrupt 12 */
#define XTB_IXR_FSYNC11_MASK	0x08000000 /**< Frame Sync Interrupt 11 */
#define XTB_IXR_FSYNC10_MASK	0x04000000 /**< Frame Sync Interrupt 10 */
#define XTB_IXR_FSYNC09_MASK	0x02000000 /**< Frame Sync Interrupt 09 */
#define XTB_IXR_FSYNC08_MASK	0x01000000 /**< Frame Sync Interrupt 08 */
#define XTB_IXR_FSYNC07_MASK	0x00800000 /**< Frame Sync Interrupt 07 */
#define XTB_IXR_FSYNC06_MASK	0x00400000 /**< Frame Sync Interrupt 06 */
#define XTB_IXR_FSYNC05_MASK	0x00200000 /**< Frame Sync Interrupt 05 */
#define XTB_IXR_FSYNC04_MASK	0x00100000 /**< Frame Sync Interrupt 04 */
#define XTB_IXR_FSYNC03_MASK	0x00080000 /**< Frame Sync Interrupt 03 */
#define XTB_IXR_FSYNC02_MASK	0x00040000 /**< Frame Sync Interrupt 02 */
#define XTB_IXR_FSYNC01_MASK	0x00020000 /**< Frame Sync Interrupt 01 */
#define XTB_IXR_FSYNC00_MASK	0x00010000 /**< Frame Sync Interrupt 00 */
#define XTB_IXR_FSYNCALL_MASK	0xFFFF0000 /**< All Frame Sync Interrupt 0-15*/

#define XTB_IXR_G_AV_MASK	0x00002000 /**< Generator Active Video Intr */
#define XTB_IXR_G_VBLANK_MASK	0x00001000 /**< Generator VBLANK Interrupt */
#define XTB_IXR_G_ALL_MASK	0x00003000 /**< All Generator interrupts */

#define XTB_IXR_D_AV_MASK	0x00000200 /**< Detector Active Video Intr */
#define XTB_IXR_D_VBLANK_MASK	0x00000100 /**< Detector VBLANK Interrupt */
#define XTB_IXR_D_ALL_MASK	0x00000300 /**< All Detector interrupts */

#define XTB_IXR_AL_MASK		0x00000080 /**< All lock */
#define XTB_IXR_ACL_MASK	0x00000040 /**< Active Chroma signal lock */
#define XTB_IXR_AVL_MASK	0x00000020 /**< Active Video Signal Lock */
#define XTB_IXR_FIL_MASK	0x00000010 /**< Field ID Signal Lock */
#define XTB_IXR_VBL_MASK	0x00000008 /**< Vertical Blank Signal Lock */
#define XTB_IXR_VSL_MASK	0x00000004 /**< Vertical Sync Signal Lock */
#define XTB_IXR_HBL_MASK	0x00000002 /**< Horizontal Blank Signal Lock */
#define XTB_IXR_HSL_MASK	0x00000001 /**< Horizontal Sync Signal Lock */
#define XTB_IXR_LOCKALL_MASK	0x000000FF /**< All Signal Lock interrupt */
#define XTB_IXR_ALLINTR_MASK	(XTB_IXR_FSYNCALL_MASK | XTB_IXR_G_ALL_MASK | \
					XTB_IXR_D_ALL_MASK | \
					XTB_IXR_LOCKALL_MASK) /**< Mask for all
								*  interrupts
								*/
/*@}*/

/**************************** Type Definitions *******************************/


/***************** Macros (Inline Functions) Definitions *********************/

/** @name Register Access Macro Definition
 *  @{
 */
#define XTimeBase_In32		Xil_In32
#define XTimeBase_Out32		Xil_Out32

/*****************************************************************************/
/**
*
* Read the given register.
*
* @param	BaseAddress is the base address of the device
* @param	RegOffset is the register offset to be read
*
* @return	The 32-bit value of the register
*
* @note
* C-style signature:
*	 u32 XTimeBase_ReadReg(u32 BaseAddress, u32 RegOffset)
*
******************************************************************************/
#define XTimeBase_ReadReg(BaseAddress, RegOffset) \
	XTimeBase_In32((BaseAddress) + (RegOffset))

/*****************************************************************************/
/**
*
* Write the given register.
*
* @param	BaseAddress is the base address of the device
* @param	RegOffset is the register offset to be written
* @param	Data is the 32-bit value to write to the register
*
* @return	None.
*
* @note
* C-style signature:
*	 void XTimeBase_WriteReg(u32 BaseAddress, u32 RegOffset, u32 Data)
*
******************************************************************************/
#define XTimeBase_WriteReg(BaseAddress, RegOffset, Data) \
	XTimeBase_Out32((BaseAddress) + (RegOffset), (Data))

/*@}*/

/************************** Function Prototypes ******************************/

#ifdef __cplusplus
}
#endif

#endif /* end of protection macro */
